/*****************************************************************************
Title: CRRCsim, the Charles River Radio Control Club Flight Simulator Project
Authors:
Jan Kansky:  Programming
Mark Drela:  Aerodynamics

Purpose:  The idea for CRRCsim is to take away any last excuse you might
have for not using a flight simulator to keep your thumbs certified for RC
flying.   It's free, it works, enjoy.  This is an open source project, so if
you don't like the way something works, help us fix it!

Thanks:
  Bruce Jackson for the LaRCsim framework.
  Flight Gear project for the sky sphere.

Contacts:
If you'd like to help with CRRCSIM, then send me an email!
    email                : kansky@ll.mit.edu
*****************************************************************************
 Joel Lienard & Lionel Cailler : Add audio interface  ("AUDIO")
*****************************************************************************/
#ifdef WIN32
 #include <windows.h>
 #include "gl.h"
 #include "glu.h"
 #include "glut.h"
 #include "tube.h"
 #include "dlportio.h"
#endif
#if defined(__APPLE__) || defined(MACOSX)
 #include <GLUT/glut.h>
 #include <GLUT/gutil.h>
 #include <GLUT/tube.h>
#endif  // __APPLE__
#ifdef linux
 #include <GL/glut.h>
 #include <GL/gutil.h>
 #include <GL/tube.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#if defined(__APPLE__) || defined(MACOSX)
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
// #include <asm/io.h>
#endif  // __APPLE__
#ifdef linux
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/mman.h>
 #include <sys/stat.h>
 #include <time.h>
 #include <sys/time.h>
 #include <asm/io.h>
#endif
#include <fcntl.h>
#include <math.h>
#include "ls_generic.h"
#include "ls_interface.h"
#include "ls_constants.h"
#include "audio_rc.h"	// AUDIO
#ifdef linux
  #include "mbuff.h"
#endif

#ifndef TRUE
 #define TRUE 1
#endif

#ifndef FALSE
 #define FALSE 0
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif


/*****************************************************************************/
#define DAVIS             1    // CRRC Davis flying field in Sudbry, Mass
#define MEDFIELD          2    // CRRC Medfield flying site in Medfield, Mass
#define CAPE_COD          3    // Slope Soaring at Cape Cod, Massachusetts 

#define FUTABA				0
#define AIRTRONICS			1
#define HITEC				2
#define JR					3
#define COCKPIT					4
#define KEYBOARD          1    // Use keyboard for fligt controls
#define MOUSE             2    // Use mouse for flight controls
#define JOYSTICK_2        3    // Use 2-axis joystick for inputs
#define JOYSTICK_3        4    // Use 3-axis joystick for inputs
#define JOYSTICK_4        5    // Use 4 axis joystick for inputs
#define RCTRAN            6    // Use LPT interrupt under Linux for inputs
#define SERIAL            7    // Use Breed Serial port interface
#define PARALLEL1         8    // Use Kansky parallel port interface
#define PARALLEL2         9    // Use Kansky parallel port interface
#define PARALLEL3         10   // Use Kansky parallel port interface
#define AUDIO             11   // Use audio interface (AUDIO code)

#define MOUSE_FOV         1
#define ARROW_KEY_FOV     2

#define SMOOTH  1
#define FLAT    2
#define FOG_R 1.0f
#define FOG_G 1.0f
#define FOG_B 1.0f
#define FOG_A 1.0f
#define occupancy_grid_size 128
#define occupancy_grid_res 100
#define area_of_influence 5
#define turb_grid_size 1024   // 1 cm resolution, 10.24m max size.

/*****************************************************************************/
int read_config_file(char *filename);
int read_airplane_data(char *datafile);
void read_airplane_file(int argc,char **argv);
void initialize_graphics(void);
void initialize_flight_model(void);
static void ShadowMatrix(float ground[4],float light[4],float shadowMat[4][4]);
void initialize_window(void);
void fgSkyVerticesInit(void);
void draw_aky(void);
void draw_medfield_field(void);
void draw_davis_field(void);
void draw_airplane(void);
void display(void);
void calculate_wind(double X_cg,double Y_cg,double H_cg,double *Vel_north,
		    double *Vel_east, double *Vel_down);
void reshape(int w, int h);
void key(unsigned char key, int px, int py);
void special(int key,int x, int y);
void mouse(int button, int state, int x, int y);
void motion(int x, int y);
void joystick(unsigned int button_mask, int x, int y, int z);
void idle(void);

/*****************************************************************************/
//Configuration Data
  int location;      // Davis field of Medfield will be the two options
  int fullscreen;    // Indicates whether to run in full screen mode
  int input_method;  //1=keyboard 2=mouse 3,4,5=joystick 6=rctran 7=Serial Port
  int radio_type=0;
  int x_resolution;  // Screen resolution to use in X direction
  int y_resolution;  // Screen resolution to use in Y direction
  int color_depth;   // Color depth to use
  int refresh_rate;  // Refresh rate to use
  float dt;          // time interval of integration of EOMs
  int initialization_time;     // Initial time when the program is started.
  int current_time;           // Current time in ms, from glut
  int time_after_last_integration;   //Time after the last integration of EOMs.
  int last_time;     // time of last iteration is ms.
  int mouse_x;       // Current mouse x position when in window
  int mouse_y;       // Current mouse y position when in window
  int window_xsize;  // Size of window in x direction
  int window_ysize;  // Size of window in y direction
  int resuming=1;    //  Will the next iteration be resuming from a pause?
  GLfloat field_of_view;  // Angular field of view in degrees.
  int zoom_control;  // What input device controls the field of view
  float shadow_matrix[4][4];  // Matrix used to calculate shadow on ground
  float slope_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
  float top_shadow_matrix[4][4];  // Matrix used to calculate shadow on slope
  float cpu_speed;   // CPU speed in Mhz, for linux rctran interface
  int shading;
  int use_textures;
  int use_mipmaps;
  int stalling=FALSE;

//Environment Data
 float wind_velocity;
 float wind_direction;
 typedef struct
  {
    float center_x_position; // Center position of thermal on ground
    float center_y_position; // Center position of thermal on ground
    float radius;            // Radius of thermal column ft 
    float boundary_thickness; // 1/e width of transition into thermal core
    float strength;          // Vertical component strength in ft/s
    float lifetime;          // How long should the thermal last in sec
    float time_expired;      // How long has the thermal been around in sec
    int xcoord;              // X coordinate in thermal occupancy grid
    int ycoord;              // Y coordinate in thermal occupancy grid
    void *next_thermal;
  } thermal;
 thermal *thermals=NULL;

  thermal *thermal_occupancy_grid[occupancy_grid_size][occupancy_grid_size];
  float thermal_density;   // How many thermals per sq mile
  float thermal_strength_mean;      // Mean strength of thermal
  float thermal_strength_sigma;     // 1 sigma variation in thermal strength
  float thermal_radius_mean;    // Mean thermal radius
  float thermal_radius_sigma; // 1 sigma thermal radius variation
  float thermal_lifetime_mean; // Average lifetime of a thermal in minutes
  float thermal_lifetime_sigma; // 1 sigma variation in lifetime in minutes
  int num_thermals;
  int training_mode;          // Draw thermals in the sky?

  float turb_x_velocity[turb_grid_size];
  float turb_y_velocity[turb_grid_size];
  float turb_z_velocity[turb_grid_size];
  float turb_x_grad[turb_grid_size];
  float turb_y_grad[turb_grid_size];
  float turb_z_grad[turb_grid_size];

// Sky Data
  float distance_to_horizon;  // The radius of the sky sphere
  float inner_vertex[12][3];  // Coordinates of sky sphere points
  float middle_vertex[12][3];
  float outer_vertex[12][3];
  float bottom_vertex[12][3];
  float inner_texture[12][2]; // Texture coordinates of sky sphere points
  float middle_texture[12][2];
  float outer_texture[12][2];
  float bottom_texture[12][2];
  float inner_color[12][4];   // Colors of sky sphere points
  float middle_color[12][4];
  float outer_color[12][4];
  char sky_texture_filename[256];
  char ground_texture_filename[256];
  char grass_texture_filename[256];
  char pine_texture_filename[256];
  char decid_texture_filename[256];
  char grass_side_texture_filename[256];
  char grass_top_texture_filename[256];
  char eastern_view_filename[256];
  char netrees_filename[256];
  char dirt_filename[256];
  char water_filename[256];
  char beachsand_filename[256];
  char scrub_filename[256];
  char scrubedge_filename[256];
  char south_filename[256];
  char hilledge_filename[256];
  char waves_filename[256];
  char sand_filename[256];
  char outhouse_filename[256];
  char freq_filename[256];
  unsigned char *sky_texture;
  int sky_texture_width;
  int sky_texture_height;
  unsigned char *ground_texture;
  int ground_texture_width;
  int ground_texture_height;
  unsigned char *grass_texture;
  int grass_texture_width;
  int grass_texture_height;
  unsigned char *grass_side_texture;
  int grass_side_texture_width;
  int grass_side_texture_height;
  unsigned char *grass_top_texture;
  int grass_top_texture_width;
  int grass_top_texture_height;
  unsigned char *eastern_view_texture;
  int eastern_view_texture_width;
  int eastern_view_texture_height;
  unsigned char *netrees_texture;
  int netrees_texture_width;
  int netrees_texture_height;
  unsigned char *dirt_texture;
  int dirt_texture_width;
  int dirt_texture_height;
  unsigned char *water_texture;
  int water_texture_width;
  int water_texture_height;
  unsigned char *beachsand_texture;
  int beachsand_texture_width;
  int beachsand_texture_height;
  unsigned char *scrub_texture;
  int scrub_texture_width;
  int scrub_texture_height;
  unsigned char *scrubedge_texture;
  int scrubedge_texture_width;
  int scrubedge_texture_height;
  unsigned char *south_texture;
  int south_texture_width;
  int south_texture_height;
  unsigned char *hilledge_texture;
  int hilledge_texture_width;
  int hilledge_texture_height;
  unsigned char *waves_texture;
  int waves_texture_width;
  int waves_texture_height;
  unsigned char *sand_texture;
  int sand_texture_width;
  int sand_texture_height;
  unsigned char *outhouse_texture;
  int outhouse_texture_width;
  int outhouse_texture_height;
  unsigned char *freq_texture;
  int freq_texture_width;
  int freq_texture_height;
  unsigned char *pine_texture;
  int pine_texture_width;
  int pine_texture_height;
  unsigned char *decid_texture;
  int decid_texture_width;
  int decid_texture_height;
  static GLuint skyTexture;          // GL sky texture handle
  static GLuint groundTexture;          // GL ground texture handle
  static GLuint grassTexture;          // GL grass texture handle
  static GLuint grassSideTexture;          // GL grass Side texture handle
  static GLuint grassTopTexture;          // GL grass Top texture handle
  static GLuint pineTexture;          // GL grass texture handle
  static GLuint decidTexture;          // GL grass texture handle
  static GLuint easternViewTexture;          // Looking to the east
  static GLuint netreesTexture;          // Looking to the east
  static GLuint dirtTexture;          // Looking to the east
  static GLuint waterTexture;          // Looking to the east
  static GLuint beachsandTexture;          // Looking to the east
  static GLuint scrubTexture;          // Looking to the east
  static GLuint scrubedgeTexture;          // Looking to the east
  static GLuint southTexture;          // Looking to the east
  static GLuint hilledgeTexture;          // Looking to the east
  static GLuint wavesTexture;          // Looking to the east
  static GLuint sandTexture;          // Looking to the east
  static GLuint outhouseTexture;          // Looking to the east
  static GLuint freqTexture;          // Looking to the east
  float INNER_RADIUS;
  float MIDDLE_RADIUS;
  float OUTER_RADIUS;
  float BOTTOM_RADIUS;
  float CENTER_ELEV;
  float INNER_ELEV;
  float MIDDLE_ELEV;
  float OUTER_ELEV;
  float BOTTOM_ELEV;

//Airplane Data
  DATA  C_ref;  //  reference chord (ft)
  DATA  B_ref;  //  reference span  (ft)
  DATA  S_ref;  //  reference area (ft^2)
  DATA  U_ref;  //  reference speed for Re-scaling of CD_prof  (ft/s)
  DATA  Alpha_0;  //  baseline alpha (rad)
  DATA  Cm_0   ;  //  baseline Cm at Alpha_0
  DATA  CL_0   ;  //  baseline CL at Alpha_0
  DATA  CL_max ;  //  positive stall limit
  DATA  CL_min ;  //  negative stall limit
  DATA  CD_prof;  //  profile CD at U_ref
  DATA  Uexp_CD;  //  for Re-scaling of CD_prof  ~ (U/U_ref)^Uexp_CD
  DATA  CL_a;  // lift-force   / alpha    ~  2 pi / (1 + 2/AR)
  DATA  Cm_a;  // pitch-moment / alpha    (pitch stability)
  DATA  CY_b;  // side-force  / sideslip
  DATA  Cl_b;  // roll-moment / sideslip (crucial for rudder-only turns)
  DATA  Cn_b;  //  yaw-moment  / sideslip (yaw stability)
  DATA  CL_q;  //  lift-force   / pitch-rate
  DATA  Cm_q;  //  pitch-moment / pitch-rate  (pitch damping)
  DATA  CY_p;  //  side-force  / roll-rate
  DATA  Cl_p;  //  roll-moment / roll-rate   (roll damping)
  DATA  Cn_p;  //  yaw-moment  / roll-rate   (yaw-roll coupling)
  DATA  CY_r;  //  side-force  / yaw-rate
  DATA  Cl_r;  //  roll-moment / yaw-rate
  DATA  Cn_r;  //  yaw-moment  / yaw-rate  (yaw damping)
  DATA  CL_de;  //  lift-force   / elevator
  DATA  Cm_de;  //  pitch-moment / elevator
  DATA  CY_dr; // side-force  / rudder
  DATA  Cl_dr; // roll-moment / rudder
  DATA  Cn_dr; // yaw-moment  / rudder
  DATA  CY_da;  // side-force  / aileron 
  DATA  Cl_da;  // roll-moment / aileron
  DATA  Cn_da;  // yaw-moment  / aileron
  DATA eta_loc;
  DATA CG_arm;
  DATA CL_drop;
  DATA CD_stall;
  DATA span_eff=1.0;  // Effective span  0.95 for most planes, 0.85 flying wing
  DATA CL_CD0=0.0;    // 0.30 for 7037, 0.15 MH32, 0.0 RG15, AGxx, power
  DATA CD_CLsq=0.0;   // 0.01 composites, 0.015 saggy ships, 0.02 beat up ship
  DATA  CD_AIsq=0.0;  // 0.01/(max_aileron)^2
  DATA CD_ELsq=0.0;   // 0.01/(max_elevator)^2 for Zagi otherwise 0

  float max_thrust;          // Maximum thrust from engine.
  float initial_altitude;    // Starting position for the airplane simulation
  float initial_velocity;    // Starting position for the airplane simulation
  float initial_theta;       // Starting pitch angle for the airplane
  float aileron_input;       // Current aileron position
  float elevator_input;      // Current elevator position
  float rudder_input;        // Current rudder position
  float throttle_input;      // Current throttle position
  float brake_input;         // Current brake position
  float flap_input;          // Current flap position
  float spoiler_input;       // Current spoiler position
  float retract_input;      // Current retract position
  int num_airfoil_points;     // Number of points on closed airfoil contour
  gleDouble **contour;        // The 2D airfoil contour
  gleDouble **cont_normal;    // Normal vectors along the contour
  int num_spanwise_points;    // Number of spanwise ribs on the wing
  gleDouble **point_array;  // Those 3D spanwise rib leading edge locations
  gleAffine *wing_scaling;   // Scaling of airfoil pattern for each rib
  float **wing_colors;      // Color at each rib location
  float wheel_radius;
  int num_wheels;
  DATA **d_wheel_rp_body_v;
  DATA *spring_constant;
  DATA *spring_damping;
  int *steering_mapping;  // Indicates which RC channel controls the steering
  int *steering_max_angle;  // Indicates maximum anglee of steering wheel
  DATA *percent_brake;
  DATA *caster_angle_rad;
  int num_extrusions;
  int dynamic_soaring;
  typedef struct
    {
      char *extrusion_name;
      gleDouble up[3];
      int num_points_on_path;
      GLfloat translate_x,translate_y,translate_z;
      GLfloat rotate_x,rotate_y,rotate_z;
      gleDouble **point_array;
      gleAffine *scaling;
      GLfloat **colors;
      int num_contour_elements;
      gleDouble **contour;
      gleDouble **contour_normal_vectors;
      void *next_extrusion;
    } extrusion;
  typedef struct
    {
      char *cylinder_name;
      GLfloat translate_x,translate_y,translate_z;
      GLfloat rotate_x,rotate_y,rotate_z;
      GLfloat radius_1,radius_2,length;
      GLfloat radial_sampling,longitudinal_sampling;
      GLfloat colors[4];
      void *next_cylinder;
    } cylinder;
  typedef struct
    {
      char *sphere_name;
      GLfloat translate_x,translate_y,translate_z;
      GLfloat rotate_x,rotate_y,rotate_z;
      GLfloat radius;
      GLfloat slices,stacks;
      GLfloat colors[4];
      void *next_sphere;
    } sphere;
  typedef struct
    {
      char *triangle_name;
      unsigned long int num_triangles;
      GLfloat translate_x,translate_y,translate_z;
      GLfloat rotate_x,rotate_y,rotate_z;
      GLfloat colors[4];
      GLfloat **vertices;
      GLfloat **normals;
      void *next_triangle;
    } triangle;

  extrusion *extrusions=NULL;
  cylinder *cylinders=NULL;
  sphere *spheres=NULL;
  triangle *triangles=NULL;
  extern GENERIC generic_;         // LaRCsim variables
#if defined(__APPLE__) || defined(MACOSX)
  volatile unsigned long long *rctran;  // Linux LPT rctran interface shmem
#endif  // __APPLE__
#ifdef linux
  volatile unsigned long long *rctran;  // Linux LPT rctran interface shmem
#endif
#ifdef WIN32
  unsigned long *rctran;
#endif
  GLUquadricObj *quadric;

unsigned int LPT_DATA;
unsigned int LPT_STATUS;
unsigned int LPT_CONTROL;
float rc_channel_values[11];
int nb_rc_channel_values=10; //AUDIO
float lpt_calibration_factor=1158726; //Number of ticks per second of the mcpu

#if defined(__APPLE__) || defined(MACOSX)
/***************************************************************************/
unsigned char inb(unsigned int PORT)
{
  return(0);
}

/***************************************************************************/
void outb(unsigned char output_value,unsigned int PORT)
{ 
  //DlPortWritePortUchar(PORT,output_value);
}

#endif  // __APPLE__

#ifdef WIN32
/***************************************************************************/
unsigned char inb(unsigned int PORT)
{
  return(DlPortReadPortUchar(PORT));
}

/***************************************************************************/
void outb(unsigned char output_value,unsigned int PORT)
{ 
  DlPortWritePortUchar(PORT,output_value);
}

#endif
/***************************************************************************/
void ask_for_data()
// Set HostBusy low to indicate data desired, inverted logic.
{
 unsigned char data;

 data=inb(LPT_CONTROL);
 outb(0x02|data,LPT_CONTROL);
}

/***************************************************************************/
void acknowledge_got_data()
// Set HostBusy high indicating that the data has been read, inverted logic.
{
 unsigned char data;

 data=inb(LPT_CONTROL);
 outb(0xFD&data,LPT_CONTROL);
}

/***************************************************************************/
void wait_until_data_ready(unsigned char *data)
// Wait until the S6 bit is brought low, indicating data is ready to be read.
{
 int num_loops;

 num_loops=0;
  do
   {
     (*data)=inb(LPT_STATUS);
     num_loops++;
   }
 while (((*data) & 0x40)!=0 && (num_loops < 25000));
 if (num_loops >=25000)
   {
     printf("Communications time-out with RC transmitter interface.\n");
   }
}


/***************************************************************************/
void wait_until_interface_is_idle(unsigned char *data)
// Wait until the S6 bit is brought high, indicating interface is idle and
// there is more data to send.
{
 int num_loops;

 num_loops=0;
  do
   {
     (*data)=inb(LPT_STATUS);
     num_loops++;
   }
 while (((*data) & 0x40)==0 && (num_loops < 25000));
 if (num_loops >=25000)
   {
     printf("Communications time-out with RC transmitter interface.\n");
   }
 (*data)=inb(LPT_STATUS);
}


/***************************************************************************/
void calibrate_parallel_interface()
//  Determine how many ticks per second elapse in the microcontroller so
// that the output can be calibrated to absolute time.
{
  unsigned char data;

#if defined(__APPLE__) || defined(MACOSX)
  struct timeval s_time,e_time;
  struct timezone tz;
#endif  // __APPLE__
#ifdef linux
  struct timeval s_time,e_time;
  struct timezone tz;
#endif
#ifdef WIN32
  _int64 frame1s, start, end;
#endif
  unsigned int int_data,new_data;
  int cycles,channel_num,current_nibble;
  float total_clocks,lpt_elapsed_time;
  int num_calibration_cycles;
  unsigned int channels[11];
  float total_clocks_array[11];
  int loop;

  for (loop=0;loop<11;loop++)
    {
      total_clocks_array[loop]=0;
      channels[loop]=0;
    }
  num_calibration_cycles=150;
  cycles=0;
  do
    {
      ask_for_data();
      wait_until_data_ready(&data);
      acknowledge_got_data();
      wait_until_interface_is_idle(&data);
    }
  while((data & 0x08) == 0);
  channel_num=1;
#if defined(__APPLE__) || defined(MACOSX)
  gettimeofday(&s_time,&tz);     // Get time that new data arrived.
#endif  // __APPLE__
#ifdef linux
  gettimeofday(&s_time,&tz);     // Get time that new data arrived.
#endif
#ifdef WIN32
  QueryPerformanceFrequency((LARGE_INTEGER*)&frame1s);
  QueryPerformanceCounter((LARGE_INTEGER*)&start);
#endif
  while (cycles<num_calibration_cycles)       // Get 150 new frames of data
    {
      ask_for_data();
      wait_until_data_ready(&data);
      acknowledge_got_data();
      if (data & 0x80)
	{
	  data&=0xBF;
	}
      else
	{
	  data|=0x40;
	}
      data=(data&0x78)>>3; 

      if (channel_num != 0)
	{
	  int_data=(unsigned int)data;
	  new_data|=(int_data<<(current_nibble*4));
	  current_nibble++;
	  if (current_nibble==3)
	    {
	      channels[channel_num-1]=new_data;
	      current_nibble=0;
	      new_data=0;
	      if (channel_num < 11)
		{
		  channel_num++;
		}
	      else
		{
		  printf("RC Transmitter Interface Read Error.\n");
		}
	      if (channel_num==10)
		{
		  total_clocks_array[0]+=channels[0];
		  total_clocks_array[1]+=channels[1];
		  total_clocks_array[2]+=channels[2];
		  total_clocks_array[3]+=channels[3];
		  total_clocks_array[4]+=channels[4];
		  total_clocks_array[5]+=channels[5];
		  total_clocks_array[6]+=channels[6];
		  total_clocks_array[7]+=channels[7];
		  total_clocks_array[8]+=channels[8];
		  total_clocks_array[9]+=channels[9];
		}
	    }
	}
      wait_until_interface_is_idle(&data);
      if (data & 0x08)
	{
	  cycles++;
	  if (cycles==num_calibration_cycles)
	    {
#if defined(__APPLE__) || defined(MACOSX)
	      gettimeofday(&e_time,&tz); 
#endif  // __APPLE__
#ifdef linux
	      gettimeofday(&e_time,&tz); 
#endif
#ifdef WIN32
		  QueryPerformanceCounter((LARGE_INTEGER*)&end);
#endif
	    }       
	  current_nibble=0;
	  new_data=0;
	  channel_num=1;
	}
    }
#if defined(__APPLE__) || defined(MACOSX)
 lpt_elapsed_time=((((float)e_time.tv_sec)*1E6+((float)e_time.tv_usec))-
		   (((float)s_time.tv_sec*1E6)+((float)s_time.tv_usec)))/1E6;
#endif  // __APPLE__
#ifdef linux
 lpt_elapsed_time=((((float)e_time.tv_sec)*1E6+((float)e_time.tv_usec))-
		   (((float)s_time.tv_sec*1E6)+((float)s_time.tv_usec)))/1E6;
#endif
#ifdef WIN32
 lpt_elapsed_time=((float)(end - start))/frame1s;
#endif

 for (loop=0;loop<9;loop++)
   {
     printf("%f ",total_clocks_array[loop]);
   }
 printf("\n");

 total_clocks=0;
 for (loop=0;loop<8;loop++)
   {
     total_clocks+=total_clocks_array[loop];
   }
 total_clocks+=total_clocks_array[9]*8;
 printf("%f\n",total_clocks);
 lpt_calibration_factor=(total_clocks/lpt_elapsed_time);
 printf("Parallel Port Micro-controller Clock Frequency: %f MHz\n",
	lpt_calibration_factor/1E6);
}

/***************************************************************************/
void init_parallel_interface(int lpt)
{
 unsigned char data;

 if (lpt == 1)
   {
    LPT_DATA=0x378;
   }
 else if (lpt==2)
   {
     LPT_DATA=0x278;
   }
 else if (lpt==3)
   {
     LPT_DATA=0x3F8;
   }
 LPT_STATUS=LPT_DATA+1;
 LPT_CONTROL=LPT_DATA+2;
#if defined(__APPLE__) || defined(MACOSX)
#endif  // __APPLE__
#ifdef linux
 if (ioperm(LPT_DATA,3,1))
  {
    printf("Sorry, you were not able to gain access to the ports\n");
    printf("You must be root to run this program\n");
    exit(1);
  }
#endif

 do
    {
     ask_for_data();
     wait_until_data_ready(&data);
     acknowledge_got_data();
     wait_until_interface_is_idle(&data);
   }
   while((data & 0x08) == 0);
 printf("Received parallel interface sync pulse.\n");
 printf("Calibrating Parallel Port micro-controller clock frequency.\n");
 calibrate_parallel_interface();
 printf("Parallel port interface calibrated.\n");
}

/***************************************************************************/
void get_data_from_parallel_interface(float *rc_channel_values)
//  Check to see if there is new data waiting on the parallel interface.
// If there is, then get a nibblet
{
  unsigned char data;
  unsigned int int_data,new_data;
  int loop,synched,channel_num,num_read,current_nibble;
  unsigned int channels[11];

  num_read=0;
  synched=0;
  current_nibble=0;
  channel_num=1;
  for (loop=0;loop<11;loop++)
    {
      channels[loop]=0;
    }
  data=inb(LPT_STATUS);
  if ((data & 0x40)!=0)
    {
      data=inb(LPT_STATUS);
      if (data & 0x08)
	{
	  current_nibble=0;
	  new_data=0;
	  channel_num=1;
	}
      do
	{
	  ask_for_data();
	  wait_until_data_ready(&data);
	  acknowledge_got_data();
	  if (data & 0x80)
	    {
	      data&=0xBF;
	    }
	  else
	    {
	      data|=0x40;
	    }
	  data=(data&0x78)>>3; 
	  if (channel_num != 0)
	    {
	      int_data=(unsigned int)data;
	      new_data|=(int_data<<(current_nibble*4));
	      current_nibble++;
	      num_read++;
	      if (current_nibble==3)
		{
		  channels[channel_num-1]=new_data;
		  rc_channel_values[channel_num-1]=((float)new_data*1000/
						    lpt_calibration_factor)-1.52;
		  current_nibble=0;
		  new_data=0;
		  if (channel_num < 11)
		    {
		      channel_num++;
		    }
		  else
		    {
		      printf("RC Transmitter Interface Read Error.\n");
		    }
		}
	    }
	  if (num_read < 30)
	    {
	      wait_until_interface_is_idle(&data);
	      if (data & 0x08)
		{
		  current_nibble=0;
		  new_data=0;
		  channel_num=1;
		  synched=1;
		}
	    }
	}
      while ((synched==0) && (num_read < 30));
    }
}

/***************************************************************************/
int read_config_file(char *filename)
{
 FILE *config_file;
 char parameter_name[256]; //  Name of parameter
 char trailing_comment[256];
 char text_value[256];
 
 config_file=fopen("crrcsim.cfg","r");
 if (config_file==NULL)
   {
     printf("Unable to open CRRCsim configuration file crrcsim.cfg\n");
     exit(0);
   }
 else
   {
     while (fscanf(config_file,"%s",(char *)parameter_name)!=EOF)
       {
	 if (strcasecmp(parameter_name,"location")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"DAVIS")==0)
	       {
		 location=DAVIS;
	       }
	     else if (strcasecmp(text_value,"MEDFIELD")==0)
	       {
		 location=MEDFIELD;
	       }
	     else if (strcasecmp(text_value,"CAPE_COD")==0)
	       {
		 location=CAPE_COD;
	       }
	   }
	 else if (strcasecmp(parameter_name,"fullscreen")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"TRUE")==0)
	       {
		 fullscreen=TRUE;
	       }
	     else if (strcasecmp(text_value,"FALSE")==0)
	       {
		 fullscreen=FALSE;
	       }
	   }
	 else if (strcasecmp(parameter_name,"training_mode")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"TRUE")==0)
	       {
		 training_mode=TRUE;
	       }
	     else if (strcasecmp(text_value,"FALSE")==0)
	       {
		 training_mode=FALSE;
	       }
	   }
	 else if (strcasecmp(parameter_name,"radio_type")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"FUTABA")==0)
	       {
		     radio_type=FUTABA;
	       }
	     else if (strcasecmp(text_value,"AIRTRONICS")==0)
	       {
		     radio_type=AIRTRONICS;
	       }
 	     else if (strcasecmp(text_value,"HITEC")==0)
	       {
		     radio_type=HITEC;
	       }
 	     else if (strcasecmp(text_value,"JR")==0)
	       {
		     radio_type=JR;
	       }
 	     else if (strcasecmp(text_value,"COCKPIT")==0)
	       {
		     radio_type=COCKPIT;
	       }
	   }
	 else if (strcasecmp(parameter_name,"input_method")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"KEYBOARD")==0)
	       {
		 input_method=KEYBOARD;
	       }
	     else if (strcasecmp(text_value,"MOUSE")==0)
	       {
		 input_method=MOUSE;
	       }
	     else if (strcasecmp(text_value,"JOYSTICK_2")==0)
	       {
		 input_method=JOYSTICK_2;
	       }
	     else if (strcasecmp(text_value,"JOYSTICK_3")==0)
	       {
		 input_method=JOYSTICK_3;
	       }
	     else if (strcasecmp(text_value,"JOYSTICK_4")==0)
	       {
		 input_method=JOYSTICK_4;
	       }
	     else if (strcasecmp(text_value,"RCTRAN")==0)
	       {
		 input_method=RCTRAN;
	       }
	     else if (strcasecmp(text_value,"SERIAL")==0)
	       {
		 input_method=SERIAL;
	       }
	     else if (strcasecmp(text_value,"PARALLEL1")==0)
	       {
		 input_method=PARALLEL1;
		 init_parallel_interface(1);
	       }
	     else if (strcasecmp(text_value,"PARALLEL2")==0)
	       {
		 input_method=PARALLEL2;
		 init_parallel_interface(2);
	       }
	     else if (strcasecmp(text_value,"PARALLEL3")==0)
	       {
		 input_method=PARALLEL3;
		 init_parallel_interface(3);
	       }
	     else if (strcasecmp(text_value,"AUDIO")==0)
	       {
		 if (audio_rc_open()==0)
                    input_method=AUDIO;
	       }
	   }
	 else if (strcasecmp(parameter_name,"x_resolution")==0)
	   {
	     fscanf(config_file,"%i %[^\n]",&x_resolution,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"y_resolution")==0)
	   {
	     fscanf(config_file,"%i %[^\n]",&y_resolution,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"color_depth")==0)
	   {
	     fscanf(config_file,"%i %[^\n]",&color_depth,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"refresh_rate")==0)
	   {
	     fscanf(config_file,"%i %[^\n]",&refresh_rate,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"dt")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&dt,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"field_of_view")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&field_of_view,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"zoom_control")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"MOUSE")==0)
	       {
		 zoom_control=MOUSE;
	       }
	     else if (strcasecmp(text_value,"KEYBOARD")==0)
	       {
		 zoom_control=KEYBOARD;
	       }
	   }
	 else if (strcasecmp(parameter_name,"cpu_speed")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&cpu_speed,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"wind_velocity")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&wind_velocity,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"wind_direction")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&wind_direction,trailing_comment);
	     wind_direction=wind_direction+270;
	     if (wind_direction>=360)
	       {
		 wind_direction-=360;
	       }
	     if (location == CAPE_COD)
	       {
		 if ((wind_direction < 360) && (wind_direction > 180))
		   {
		     if (wind_direction != 270)
		       {
			 printf("Off normal winds not supported yet.\n");
		       }
		     wind_direction=270;
		     printf("Sloping.\n");
		     dynamic_soaring=FALSE;
		   }
		 else
		   {
		     if (wind_direction != 90)
		       {
			 printf("Off normal winds not supported yet.\n");
		       }
		     wind_direction=90;
		     printf("Dynamic soaring.\n");
		     dynamic_soaring=TRUE;
		   }
	       }

	   }
	 else if (strcasecmp(parameter_name,"thermal_strength_mean")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_strength_mean,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_strength_sigma")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_strength_sigma,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_radius_mean")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_radius_mean,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_radius_sigma")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_radius_sigma,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_lifetime_mean")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_lifetime_mean,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_lifetime_sigma")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_lifetime_sigma,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"thermal_density")==0)
	   {
	     fscanf(config_file,"%f %[^\n]",&thermal_density,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"sky_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",sky_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"ground_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",ground_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"grass_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",grass_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"grasss_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",grass_side_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"grasst_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",grass_top_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"eastern_view")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",eastern_view_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"netrees_tex")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",netrees_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"dirt_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",dirt_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"water_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",water_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"beachsand_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",beachsand_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"scrub_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",scrub_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"scrubedge_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",scrubedge_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"south_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",south_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"hilledge_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",hilledge_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"waves_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",waves_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"sand_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",sand_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"outhouse_tex")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",outhouse_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"freq_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",freq_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"pine_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",pine_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"decid_texture")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",decid_texture_filename,trailing_comment);
	   }
	 else if (strcasecmp(parameter_name,"shading")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"SMOOTH")==0)
	       {
		 shading=SMOOTH;
	       }
	     else if (strcasecmp(text_value,"FLAT")==0)
	       {
		 shading=FLAT;
	       }
	   }
	 else if (strcasecmp(parameter_name,"use_textures")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"TRUE")==0)
	       {
		 use_textures=TRUE;
	       }
	     else if (strcasecmp(text_value,"FALSE")==0)
	       {
		 use_textures=FALSE;
	       }
	   }
	 else if (strcasecmp(parameter_name,"use_mipmaps")==0)
	   {
	     fscanf(config_file,"%s %[^\n]",text_value,trailing_comment);
	     if (strcasecmp(text_value,"TRUE")==0)
	       {
		 use_mipmaps=TRUE;
	       }
	     else if (strcasecmp(text_value,"FALSE")==0)
	       {
		 use_mipmaps=FALSE;
	       }
	   }
       }
   }
 printf("Read in CRRCsim configuration file sucessfully.\n");
 return(0);
}


/***************************************************************************/
int read_airplane_data(char *datafile)
//  Read in datafile describing the particular aircraft currently in use.
{
  FILE *airplane_data;
  char parameter_name[256]; //  Name of parameter
  char trailing_comment[256];
  double parameter_value;
  int done_parsing_extrusion; // Hit the end_extrusion keyword
  int done_parsing_cylinder; // Hit the end_cylinder keyword
  int done_parsing_sphere; // Hit the end_sphere keyword
  int done_parsing_triangles; // Hit the end_triangle keyword
  int done_parsing_gear; // Hit the end_gear keyword
  extrusion *extrusion_ptr; // Pointer to the current extrusion being read
  extrusion *last_extrusion=NULL; //Pointer to the last filled extrusion struct
  cylinder *cylinder_ptr;      // Pointer to the current cylinder being read
  cylinder *last_cylinder=NULL;  // Pointer to the last filled cylinder struct
  sphere *sphere_ptr;      // Pointer to the current sphere being read
  sphere *last_sphere=NULL;  // Pointer to the last filled sphere struct
  triangle *triangle_ptr=NULL;  //Pointer to the current triangles struct
  triangle *last_triangle=NULL; // Pointer to the last filled triangle struct
  int loop;                   // Loop counter for extrusion parsing
  float p1,p2,p3,p4,p5,p6;    // Temporary variables used to read in file
  float x_component,y_component,magnitude;
  float dx1,dy1,dz1,dx2,dy2,dz2,normx,normy,normz,amplitude;

  airplane_data=fopen(datafile,"r");
  if (airplane_data == NULL)
   {
    printf("Unable to open airplane specification file %s.\n",datafile);
    return(-1);
   }
  else
   {
    while (fscanf(airplane_data,"%s %lf %[^\n]",(char *)parameter_name,
           &parameter_value,trailing_comment)!=EOF)
     {
       printf("%s\n",parameter_name);
      if (strcasecmp(parameter_name,"Mass")==0)
       Mass=parameter_value;
      else if (strcasecmp(parameter_name,"I_xx")==0)
       I_xx=parameter_value;
      else if (strcasecmp(parameter_name,"I_yy")==0)
       I_yy=parameter_value;
      else if (strcasecmp(parameter_name,"I_zz")==0)
       I_zz=parameter_value;
      else if (strcasecmp(parameter_name,"I_xz")==0)
       I_xz=parameter_value;
      else if (strcasecmp(parameter_name,"Theta")==0)
       Theta=parameter_value;
      else if (strcasecmp(parameter_name,"Altitude")==0)
       Altitude=parameter_value;
      else if (strcasecmp(parameter_name,"C_ref")==0)
       C_ref=parameter_value;
      else if (strcasecmp(parameter_name,"B_ref")==0)
       B_ref=parameter_value;
      else if (strcasecmp(parameter_name,"S_ref")==0)
       S_ref=parameter_value;
      else if (strcasecmp(parameter_name,"U_ref")==0)
       U_ref=parameter_value;
      else if (strcasecmp(parameter_name,"Alpha_0")==0)
       Alpha_0=parameter_value;
      else if (strcasecmp(parameter_name,"Cm_0")==0)
       Cm_0=parameter_value;
      else if (strcasecmp(parameter_name,"CL_0")==0)
       CL_0=parameter_value;
      else if (strcasecmp(parameter_name,"CL_max")==0)
       CL_max=parameter_value;
      else if (strcasecmp(parameter_name,"CL_min")==0)
       CL_min=parameter_value;
      else if (strcasecmp(parameter_name,"CD_prof")==0)
       CD_prof=parameter_value;
      else if (strcasecmp(parameter_name,"Uexp_CD")==0)
       Uexp_CD=parameter_value;
      else if (strcasecmp(parameter_name,"CL_a")==0)
       CL_a=parameter_value;
      else if (strcasecmp(parameter_name,"Cm_a")==0)
       Cm_a=parameter_value;
      else if (strcasecmp(parameter_name,"CY_b")==0)
       CY_b=parameter_value;
      else if (strcasecmp(parameter_name,"Cl_b")==0)
       Cl_b=parameter_value;
      else if (strcasecmp(parameter_name,"Cn_b")==0)
       Cn_b=parameter_value;
      else if (strcasecmp(parameter_name,"CL_q")==0)
       CL_q=parameter_value;
      else if (strcasecmp(parameter_name,"Cm_q")==0)
       Cm_q=parameter_value;
      else if (strcasecmp(parameter_name,"CY_p")==0)
       CY_p=parameter_value;
      else if (strcasecmp(parameter_name,"Cl_p")==0)
       Cl_p=parameter_value;
      else if (strcasecmp(parameter_name,"Cn_p")==0)
       Cn_p=parameter_value;
      else if (strcasecmp(parameter_name,"CY_r")==0)
       CY_r=parameter_value;
      else if (strcasecmp(parameter_name,"Cl_r")==0)
       Cl_r=parameter_value;
      else if (strcasecmp(parameter_name,"Cn_r")==0)
       Cn_r=parameter_value;
      else if (strcasecmp(parameter_name,"CL_de")==0)
       CL_de=parameter_value;
      else if (strcasecmp(parameter_name,"Cm_de")==0)
       Cm_de=parameter_value;
      else if (strcasecmp(parameter_name,"CY_dr")==0)
       CY_dr=parameter_value;
      else if (strcasecmp(parameter_name,"Cl_dr")==0)
       Cl_dr=parameter_value;
      else if (strcasecmp(parameter_name,"Cn_dr")==0)
       Cn_dr=parameter_value;
      else if (strcasecmp(parameter_name,"CY_da")==0)
       CY_da=parameter_value;
      else if (strcasecmp(parameter_name,"Cl_da")==0)
       Cl_da=parameter_value;
      else if (strcasecmp(parameter_name,"Cn_da")==0)
       Cn_da=parameter_value;
      else if (strcasecmp(parameter_name,"eta_loc")==0)
       eta_loc=parameter_value;
      else if (strcasecmp(parameter_name,"span_eff")==0)
       span_eff=parameter_value;
      else if (strcasecmp(parameter_name,"CL_CD0")==0)
       CL_CD0=parameter_value;
      else if (strcasecmp(parameter_name,"CD_CLsq")==0)
       CD_CLsq=parameter_value;
      else if (strcasecmp(parameter_name,"CD_AIsq")==0)
       CD_AIsq=parameter_value;
      else if (strcasecmp(parameter_name,"CD_ELsq")==0)
       CD_ELsq=parameter_value;
      else if (strcasecmp(parameter_name,"CG_arm")==0)
       CG_arm=parameter_value;
      else if (strcasecmp(parameter_name,"CL_drop")==0)
       CL_drop=parameter_value;
      else if (strcasecmp(parameter_name,"CD_stall")==0)
       CD_stall=parameter_value;
      else if (strcasecmp(parameter_name,"max_thrust")==0)
	max_thrust=parameter_value;
      else if (strcasecmp(parameter_name,"initial_altitude")==0)
	initial_altitude=parameter_value;
      else if (strcasecmp(parameter_name,"initial_velocity")==0)
	initial_velocity=parameter_value;
      else if (strcasecmp(parameter_name,"initial_theta")==0)
	initial_theta=parameter_value;
      else if (strcasecmp(parameter_name,"gear")==0)
	{
	  printf("In gear\n");
	  num_wheels=(int)parameter_value;
	  d_wheel_rp_body_v=(DATA **)malloc(num_wheels*sizeof(DATA *));
	  d_wheel_rp_body_v[0]=(DATA *)malloc(num_wheels*sizeof(DATA)*3);
	  for (loop=1;loop<num_wheels;loop++)
	    {
	      d_wheel_rp_body_v[loop]=d_wheel_rp_body_v[0] +loop*3;
	    }
	  spring_constant=malloc(num_wheels*sizeof(DATA));
	  spring_damping=malloc(num_wheels*sizeof(DATA));
	  percent_brake=malloc(num_wheels*sizeof(DATA));
	  steering_mapping=malloc(num_wheels*sizeof(int));
	  steering_max_angle=malloc(num_wheels*sizeof(DATA));
	  caster_angle_rad=malloc(num_wheels*sizeof(DATA));
	  done_parsing_gear=FALSE;
	  while(!done_parsing_gear)
	    {
    	      fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
		     trailing_comment); 
	      printf("     %s\n",parameter_name);
	      if (strcasecmp(parameter_name,"locations")==0)
		{
		  for (loop=0;loop<num_wheels;loop++)
		    {
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      d_wheel_rp_body_v[loop][0]=(DATA)p1;
		      d_wheel_rp_body_v[loop][1]=(DATA)p2;
		      d_wheel_rp_body_v[loop][2]=(DATA)p3;
		    }
		}
	      else if (strcasecmp(parameter_name,"springiness")==0)
		{
		  for (loop=0;loop<num_wheels;loop++)
		    {
		      fscanf(airplane_data,"%f",&p1);
		      spring_constant[loop]=(DATA)p1;
		    }
		}
	      else if (strcasecmp(parameter_name,"damping")==0)
		{
		  for (loop=0;loop<num_wheels;loop++)
		    {
		      fscanf(airplane_data,"%f",&p1);
		      spring_damping[loop]=(DATA)p1;
		    }
		}
	      else if (strcasecmp(parameter_name,"braking")==0)
		{
		  for (loop=0;loop<num_wheels;loop++)
		    {
		      fscanf(airplane_data,"%f",&p1);
		      percent_brake[loop]=(DATA)p1;
		    }
		}
	      else if (strcasecmp(parameter_name,"steerability")==0)
		{
		  for (loop=0;loop<num_wheels;loop++)
		    {
		      fscanf(airplane_data,"%f %f",&p1,&p2);
		      steering_mapping[loop]=(int)p1;
		      steering_max_angle[loop]=(DATA)p2;
		      caster_angle_rad[loop]=0;
		    }
		}
	      else if (strcasecmp(parameter_name,"end_gear")==0)
		{
		  done_parsing_gear=TRUE;
		}
	    }

	}
      else if (strcasecmp(parameter_name,"triangles")==0)
	{
	  printf("In triangles\n");
	  triangle_ptr=malloc(sizeof(triangle));
	  triangle_ptr->triangle_name=malloc((strlen(trailing_comment)+1)*
                                             sizeof(char));
	  strcpy(triangle_ptr->triangle_name,trailing_comment);
	  triangle_ptr->num_triangles=(unsigned long int)parameter_value;
	  triangle_ptr->vertices=(GLfloat **)malloc(triangle_ptr->
	    num_triangles*sizeof(GLfloat *));
	  triangle_ptr->normals=(GLfloat **)malloc(triangle_ptr->
	    num_triangles*sizeof(GLfloat *));
	  triangle_ptr->vertices[0]=(GLfloat *)malloc(triangle_ptr->
            num_triangles*sizeof(GLfloat)*9);
	  triangle_ptr->normals[0]=(GLfloat *)malloc(triangle_ptr->
            num_triangles*sizeof(GLfloat)*3);
	  for (loop=1;loop<(int)triangle_ptr->num_triangles;loop++)
	    {
	      triangle_ptr->vertices[loop]=triangle_ptr->vertices[0]+ loop*9;
	      triangle_ptr->normals[loop]=triangle_ptr->normals[0]+ loop*3;
	    }
	  triangle_ptr->next_triangle=NULL;
	  if (last_triangle==NULL)
	    triangles=triangle_ptr;
	  else
	    last_triangle->next_triangle=triangle_ptr;
          last_triangle=triangle_ptr;
	  done_parsing_triangles=FALSE;
	  while(!done_parsing_triangles)
	    {
	      fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
		     trailing_comment); 
	      printf("     %s\n",parameter_name);
	      if (strcasecmp(parameter_name,"translate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  triangle_ptr->translate_x=p1;
		  triangle_ptr->translate_y=p2;
		  triangle_ptr->translate_z=p3;
		}      
	      else if (strcasecmp(parameter_name,"rotate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  triangle_ptr->rotate_x=p1;
		  triangle_ptr->rotate_y=p2;
		  triangle_ptr->rotate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"color")==0)
		{
		  fscanf(airplane_data,"%f %f %f %f",&p1,&p2,&p3,&p4);
		  triangle_ptr->colors[0]=p1;
		  triangle_ptr->colors[1]=p2;
		  triangle_ptr->colors[2]=p3;
		  triangle_ptr->colors[3]=p4;
		}
	      else if (strcasecmp(parameter_name,"vertices")==0)
		{
		  for (loop=0;loop<(int)triangle_ptr->num_triangles;loop++)
		    {
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      triangle_ptr->vertices[loop][0]=p1;
		      triangle_ptr->vertices[loop][1]=p2;
		      triangle_ptr->vertices[loop][2]=p3;
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      triangle_ptr->vertices[loop][3]=p1;
		      triangle_ptr->vertices[loop][4]=p2;
		      triangle_ptr->vertices[loop][5]=p3;
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      triangle_ptr->vertices[loop][6]=p1;
		      triangle_ptr->vertices[loop][7]=p2;
		      triangle_ptr->vertices[loop][8]=p3;
		      
		      dx1=triangle_ptr->vertices[loop][3]-
			  triangle_ptr->vertices[loop][0];
		      dy1=triangle_ptr->vertices[loop][1]-
			  triangle_ptr->vertices[loop][4];
		      dz1=triangle_ptr->vertices[loop][2]-
			  triangle_ptr->vertices[loop][5];
		      dx2=triangle_ptr->vertices[loop][3]-
			  triangle_ptr->vertices[loop][6];
		      dy2=triangle_ptr->vertices[loop][4]-
			  triangle_ptr->vertices[loop][7];
		      dz2=triangle_ptr->vertices[loop][5]-
			  triangle_ptr->vertices[loop][8];
		      normx=dy1*dz2-dz1*dy2;
		      normy=dz1*dx2-dx1*dz2;
		      normz=dx1*dy2-dy1*dx2;
		      amplitude=sqrt(normx*normx+normy*normy+normz*normz);
		      triangle_ptr->normals[loop][0]=normx/amplitude;
		      triangle_ptr->normals[loop][1]=normy/amplitude;
		      triangle_ptr->normals[loop][2]=normz/amplitude;
		    }
		}
	      else if (strcasecmp(parameter_name,"end_triangles")==0)
		{
		  done_parsing_triangles=TRUE;
		} 
	    }
	}
      else if (strcasecmp(parameter_name,"sphere")==0)
	{
	  printf("In sphere\n");
	  sphere_ptr=malloc(sizeof(sphere));
	  sphere_ptr->sphere_name=malloc((strlen(trailing_comment)+1)*
                                             sizeof(char));
	  strcpy(sphere_ptr->sphere_name,trailing_comment);
	  sphere_ptr->next_sphere=NULL;
	  if (last_sphere==NULL)
	    spheres=sphere_ptr;
	  else
	    last_sphere->next_sphere=sphere_ptr;
          last_sphere=sphere_ptr;
	  done_parsing_sphere=FALSE;
	  while(!done_parsing_sphere)
	    {
    	      fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
		     trailing_comment); 
	      printf("     %s\n",parameter_name);
	      if (strcasecmp(parameter_name,"translate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  sphere_ptr->translate_x=p1;
		  sphere_ptr->translate_y=p2;
		  sphere_ptr->translate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"rotate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  sphere_ptr->rotate_x=p1;
		  sphere_ptr->rotate_y=p2;
		  sphere_ptr->rotate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"dimension")==0)
		{
		  fscanf(airplane_data,"%f",&p1);
		  sphere_ptr->radius=p1;
		}
	      else if (strcasecmp(parameter_name,"grid")==0)
		{
		  fscanf(airplane_data,"%f %f",&p1,&p2);
		  sphere_ptr->slices=(int)p1;
		  sphere_ptr->stacks=(int)p2;
		}
	      else if (strcasecmp(parameter_name,"color")==0)
		{
		  fscanf(airplane_data,"%f %f %f %f",&p1,&p2,&p3,&p4);
		  sphere_ptr->colors[0]=p1;
		  sphere_ptr->colors[1]=p2;
		  sphere_ptr->colors[2]=p3;
		  sphere_ptr->colors[3]=p4;
		}	
	      else if (strcasecmp(parameter_name,"end_sphere")==0)
		{
		  done_parsing_sphere=TRUE;
		}     
	    }
	}
      else if (strcasecmp(parameter_name,"cylinder")==0)
	{
	  printf("In cylinder\n");
	  cylinder_ptr=malloc(sizeof(cylinder));
	  cylinder_ptr->cylinder_name=malloc((strlen(trailing_comment)+1)*
                                             sizeof(char));
	  strcpy(cylinder_ptr->cylinder_name,trailing_comment);
	  cylinder_ptr->next_cylinder=NULL;
	  if (last_cylinder==NULL)
	    cylinders=cylinder_ptr;
	  else
	    last_cylinder->next_cylinder=cylinder_ptr;
          last_cylinder=cylinder_ptr;
	  done_parsing_cylinder=FALSE;
	  while(!done_parsing_cylinder)
	    {
    	      fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
		     trailing_comment); 
	      printf("     %s\n",parameter_name);
	      if (strcasecmp(parameter_name,"translate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  cylinder_ptr->translate_x=p1;
		  cylinder_ptr->translate_y=p2;
		  cylinder_ptr->translate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"rotate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  cylinder_ptr->rotate_x=p1;
		  cylinder_ptr->rotate_y=p2;
		  cylinder_ptr->rotate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"dimension")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  cylinder_ptr->radius_1=p1;
		  cylinder_ptr->radius_2=p2;
		  cylinder_ptr->length=p3;
		}
	      else if (strcasecmp(parameter_name,"grid")==0)
		{
		  fscanf(airplane_data,"%f %f",&p1,&p2);
		  cylinder_ptr->radial_sampling=(int)p1;
		  cylinder_ptr->longitudinal_sampling=(int)p2;
		}
	      else if (strcasecmp(parameter_name,"color")==0)
		{
		  fscanf(airplane_data,"%f %f %f %f",&p1,&p2,&p3,&p4);
		  cylinder_ptr->colors[0]=p1;
		  cylinder_ptr->colors[1]=p2;
		  cylinder_ptr->colors[2]=p3;
		  cylinder_ptr->colors[3]=p4;
		}	
	      else if (strcasecmp(parameter_name,"end_cylinder")==0)
		{
		  done_parsing_cylinder=TRUE;
		}     
	    }
	}
      else if (strcasecmp(parameter_name,"extrusion")==0)
	{
	  printf("In extrusion.\n");
	  extrusion_ptr=malloc(sizeof(extrusion));
	  extrusion_ptr->num_points_on_path=(int)parameter_value+2;
	  extrusion_ptr->extrusion_name=malloc((strlen(trailing_comment)+1)*
					       sizeof(char));
	  strcpy(extrusion_ptr->extrusion_name,trailing_comment);
	  extrusion_ptr->point_array=(gleDouble **)malloc(extrusion_ptr->
	    num_points_on_path*sizeof(gleDouble *));
	  extrusion_ptr->point_array[0]=(gleDouble *)malloc(3*extrusion_ptr->
	    num_points_on_path*sizeof(gleDouble));
	  extrusion_ptr->colors=(GLfloat **)malloc(extrusion_ptr->
            num_points_on_path*sizeof(GLfloat *));
	  extrusion_ptr->colors[0]=(GLfloat *)malloc(3*extrusion_ptr->
            num_points_on_path*sizeof(GLfloat));
	  extrusion_ptr->scaling=(gleAffine *)malloc(extrusion_ptr->
	    num_points_on_path*sizeof(gleAffine));
	  for (loop=1;loop<extrusion_ptr->num_points_on_path;loop++)
	    {
	      extrusion_ptr->point_array[loop]=extrusion_ptr->point_array[0]+
                    loop*3;
	      extrusion_ptr->colors[loop]=extrusion_ptr->colors[0]+
                    loop*3;
	    }
	  extrusion_ptr->next_extrusion=NULL;
	  if (last_extrusion==NULL)    // If this is the first parsed extrusion
	    extrusions=extrusion_ptr;  
	  else
	    last_extrusion->next_extrusion=extrusion_ptr;
	  last_extrusion=extrusion_ptr;
	  done_parsing_extrusion=FALSE;
	  while(!done_parsing_extrusion)
	    {
	      fscanf(airplane_data,"%s %[^\n]",(char *)parameter_name,
		     trailing_comment);
	      printf("     %s\n",parameter_name);
	      if (strcasecmp(parameter_name,"translate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  extrusion_ptr->translate_x=p1;
		  extrusion_ptr->translate_y=p2;
		  extrusion_ptr->translate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"rotate")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  extrusion_ptr->rotate_x=p1;
		  extrusion_ptr->rotate_y=p2;
		  extrusion_ptr->rotate_z=p3;
		}
	      else if (strcasecmp(parameter_name,"up_vector")==0)
		{
		  fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		  extrusion_ptr->up[0]=p1;
		  extrusion_ptr->up[1]=p2;
		  extrusion_ptr->up[2]=p3;
		}
	      else if (strcasecmp(parameter_name,"path")==0)
		{
		  for (loop=1;loop<extrusion_ptr->num_points_on_path-1;loop++)
		    {
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      extrusion_ptr->point_array[loop][0]=p1;
		      extrusion_ptr->point_array[loop][1]=p2;
		      extrusion_ptr->point_array[loop][2]=p3;
		    }
		  extrusion_ptr->point_array[0][0]=extrusion_ptr->
                                                      point_array[1][0];
		  extrusion_ptr->point_array[0][1]=extrusion_ptr->
		                                      point_array[1][1];
		  extrusion_ptr->point_array[0][2]=extrusion_ptr->
		                                      point_array[1][2];
		  extrusion_ptr->point_array[extrusion_ptr->
                    num_points_on_path-1][0]=extrusion_ptr->
                    point_array[extrusion_ptr->num_points_on_path-2][0];
		  extrusion_ptr->point_array[extrusion_ptr->
                    num_points_on_path-1][1]=extrusion_ptr->
                    point_array[extrusion_ptr->num_points_on_path-2][1];
		  extrusion_ptr->point_array[extrusion_ptr->
                    num_points_on_path-1][2]=extrusion_ptr->
                    point_array[extrusion_ptr->num_points_on_path-2][2];
		}
	      else if (strcasecmp(parameter_name,"colors")==0)
		{
		  for (loop=1;loop<extrusion_ptr->num_points_on_path-1;loop++)
		    {
		      fscanf(airplane_data,"%f %f %f",&p1,&p2,&p3);
		      extrusion_ptr->colors[loop][0]=p1;
		      extrusion_ptr->colors[loop][1]=p2;
		      extrusion_ptr->colors[loop][2]=p3;
		    }
		  extrusion_ptr->colors[0][0]=extrusion_ptr->
                                              colors[1][0];
		  extrusion_ptr->colors[0][1]=extrusion_ptr->
		                              colors[1][1];
		  extrusion_ptr->colors[0][2]=extrusion_ptr->
		                              colors[1][2];
		  extrusion_ptr->colors[extrusion_ptr->
                    num_points_on_path-1][0]=extrusion_ptr->
                    colors[extrusion_ptr->num_points_on_path-2][0];
		  extrusion_ptr->colors[extrusion_ptr->
                    num_points_on_path-1][1]=extrusion_ptr->
                    colors[extrusion_ptr->num_points_on_path-2][1];
		  extrusion_ptr->colors[extrusion_ptr->
                    num_points_on_path-1][2]=extrusion_ptr->
                    colors[extrusion_ptr->num_points_on_path-2][2];
		}
	      else if (strcasecmp(parameter_name,"scaling")==0)
		{
		  for (loop=1;loop<extrusion_ptr->num_points_on_path-1;loop++)
		    {
		      fscanf(airplane_data,"%f %f %f %f %f %f",&p1,&p2,&p3,&p4,
                                                               &p5,&p6);
		      extrusion_ptr->scaling[loop][0][0]=p1;
		      extrusion_ptr->scaling[loop][0][1]=p2;
		      extrusion_ptr->scaling[loop][0][2]=p3;
		      extrusion_ptr->scaling[loop][1][0]=p4;
		      extrusion_ptr->scaling[loop][1][1]=p5;
		      extrusion_ptr->scaling[loop][1][2]=p6;
		    }
		  extrusion_ptr->scaling[0][0][0]=extrusion_ptr->
                                               scaling[1][0][0];
		  extrusion_ptr->scaling[0][0][1]=extrusion_ptr->
		                               scaling[1][0][1];
		  extrusion_ptr->scaling[0][0][2]=extrusion_ptr->
		                               scaling[1][0][2];
		  extrusion_ptr->scaling[0][1][0]=extrusion_ptr->
                                               scaling[1][1][0];
		  extrusion_ptr->scaling[0][1][1]=extrusion_ptr->
		                               scaling[1][1][1];
		  extrusion_ptr->scaling[0][1][2]=extrusion_ptr->
		                               scaling[1][1][2];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][0][0]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][0][0];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][0][1]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][0][1];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][0][2]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][0][2];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][1][0]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][1][0];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][1][1]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][1][1];
		  extrusion_ptr->scaling[extrusion_ptr->
                    num_points_on_path-1][1][2]=extrusion_ptr->
                    scaling[extrusion_ptr->num_points_on_path-2][1][2];
		}
	      else if (strcasecmp(parameter_name,"section")==0)
		{
		  fscanf(airplane_data,"%f",&p1);
		  extrusion_ptr->num_contour_elements=(int)p1;
		  extrusion_ptr->contour=(gleDouble **)malloc(extrusion_ptr->
                        num_contour_elements*sizeof(gleDouble *));
		  extrusion_ptr->contour[0]=(gleDouble *)malloc(extrusion_ptr->
			num_contour_elements*2*sizeof(gleDouble));
		  extrusion_ptr->contour_normal_vectors=(gleDouble **)
                        malloc(extrusion_ptr->num_contour_elements*
	   	        sizeof(gleDouble *));
		  extrusion_ptr->contour_normal_vectors[0]=(gleDouble *)
                        malloc(extrusion_ptr->num_contour_elements*2*
	   	        sizeof(gleDouble));

		  for (loop=1;loop<extrusion_ptr->num_contour_elements;loop++)
		    {
		      extrusion_ptr->contour[loop]=extrusion_ptr->contour[0]+
			2*loop;
		      extrusion_ptr->contour_normal_vectors[loop]=
                        extrusion_ptr->contour_normal_vectors[0]+
			2*loop;
		    }
		  for (loop=0;loop<extrusion_ptr->num_contour_elements;loop++)
		    {
		      fscanf(airplane_data,"%f %f",&p1,&p2);
		      extrusion_ptr->contour[loop][0]=p1;
		      extrusion_ptr->contour[loop][1]=p2;
		    }
		  for (loop=2;loop<extrusion_ptr->num_contour_elements;loop++)
		    {
		      x_component=(extrusion_ptr->contour[loop][0]-
			extrusion_ptr->contour[loop-1][0]);
		      y_component=(extrusion_ptr->contour[loop][1]-
			extrusion_ptr->contour[loop-1][1]);
		      magnitude=sqrt(x_component*x_component+
                                     y_component*y_component);
		      extrusion_ptr->contour_normal_vectors[loop-1][0]=
                        y_component/magnitude;
		      extrusion_ptr->contour_normal_vectors[loop-1][1]=
                        -1*x_component/magnitude;

		      x_component=(extrusion_ptr->contour[loop-1][0]-
			extrusion_ptr->contour[loop-2][0]);
		      y_component=(extrusion_ptr->contour[loop-1][1]-
			extrusion_ptr->contour[loop-2][1]);
		      magnitude=sqrt(x_component*x_component+
                                     y_component*y_component);

		      extrusion_ptr->contour_normal_vectors[loop-1][0]+=
                        y_component/magnitude;
		      extrusion_ptr->contour_normal_vectors[loop-1][1]+=
                        -1*x_component/magnitude;
		      magnitude=sqrt(
                         extrusion_ptr->contour_normal_vectors[loop-1][0]*
                         extrusion_ptr->contour_normal_vectors[loop-1][0]+
                         extrusion_ptr->contour_normal_vectors[loop-1][1]*
			 extrusion_ptr->contour_normal_vectors[loop-1][1]);
		      extrusion_ptr->contour_normal_vectors[loop-1][0]/=
                         magnitude;
		      extrusion_ptr->contour_normal_vectors[loop-1][1]/=
			 magnitude;
		    }
		  x_component=(extrusion_ptr->contour[1][0]-
			       extrusion_ptr->contour[0][0]);
		  y_component=(extrusion_ptr->contour[1][1]-
			       extrusion_ptr->contour[0][1]);
		  magnitude=sqrt(x_component*x_component+
				 y_component*y_component);
		  extrusion_ptr->contour_normal_vectors[0][0]=y_component/
		    magnitude;
		  extrusion_ptr->contour_normal_vectors[0][1]= -1*x_component/
		    magnitude;

		  x_component=(extrusion_ptr->contour[extrusion_ptr->
			      num_contour_elements-1][0]-
                              extrusion_ptr->contour[extrusion_ptr->
                              num_contour_elements-2][0]);
		  y_component=(extrusion_ptr->contour[extrusion_ptr->
			      num_contour_elements-1][1]-
			       extrusion_ptr->contour[extrusion_ptr->
                              num_contour_elements-2][1]);
		  magnitude=sqrt(x_component*x_component+
				 y_component*y_component);
		  extrusion_ptr->contour_normal_vectors[extrusion_ptr->
			      num_contour_elements-1][0]=y_component/
		    magnitude;
		  extrusion_ptr->contour_normal_vectors[extrusion_ptr->
			      num_contour_elements-1][1]= -1*x_component/
		    magnitude;
		}
	      else if (strcasecmp(parameter_name,"end_extrusion")==0)
		{
		  done_parsing_extrusion=TRUE;
		}
	    }
	}
     }
    fclose(airplane_data);
    return(0);
   }
}

/*****************************************************************************/
void read_airplane_file(int argc,char **argv)
// Read in the airplane parameters from the file specified in filename
{
  int res;

  if (argc == 1)   // No airplane filname was passed in to crrcsim
    {
      res=read_airplane_data("default.air");
    }
  else
    {
      res=read_airplane_data(argv[1]);
    }
  if (res>0)
    {
      printf("Something is wrong with the input file on line %i",res);
      exit(0);
    }
  else if (res == -1)
    {
      exit(0);
    }
  else
    {
      printf("Read in airplane data file successfully.\n");
    }
}

/*****************************************************************************/
void initialize_graphics(void)
{
 distance_to_horizon=8000;

 fgSkyVerticesInit();
}

/*****************************************************************************/
void initialize_flight_model(void)
{
  Altitude=initial_altitude;
  if (location==CAPE_COD)
    {
     Altitude=initial_altitude+100;
    }
  P_body=0.0;
  Q_body=0.0;
  R_body=0.0;
  Phi=0.0;
  Theta=0.0;
  Psi=(wind_direction*M_PI/180)+M_PI;
  Alpha=0;
  Beta=0;
  Alpha_dot=0;
  D_cg_north_of_rwy=0;
  D_cg_east_of_rwy=0;
  D_cg_above_rwy=0;
  Beta_dot=0;
  if (location== CAPE_COD)
    {
      Latitude=+0.000002;
      Longitude=-0.0000005;
    }
  else
    {
      Latitude=0.0;
      Longitude=0.000001;
    }
  X_cg_rwy=0;
  Y_cg_rwy=0;
  H_cg_rwy=initial_altitude;
  V_north=0;
  V_dot_north=0;
  V_east=initial_velocity;
  V_dot_east=0;
  V_down=0;
  V_dot_down=0;
  V_north_airmass=0;
  V_east_airmass=0;
  V_down_airmass=0;
  Dx_pilot=0;
  Dy_pilot=0;
  Dz_pilot=0;
  Dx_cg=0;
  Dy_cg=0;
  Dz_cg=0;
  CD_stall=0.05;
  ls_toplevel_init(dt);
}

/*****************************************************************************/
static void ShadowMatrix(float ground[4],float light[4],float shadowMat[4][4])
{
  float dot;
/*  float shadowMat[4][4]; */

  dot = ground[0] * light[0] +
    ground[1] * light[1] +
    ground[2] * light[2] +
    ground[3] * light[3];

  shadowMat[0][0] = dot - light[0] * ground[0];
  shadowMat[1][0] = 0.0 - light[0] * ground[1];
  shadowMat[2][0] = 0.0 - light[0] * ground[2];
  shadowMat[3][0] = 0.0 - light[0] * ground[3];

  shadowMat[0][1] = 0.0 - light[1] * ground[0];
  shadowMat[1][1] = dot - light[1] * ground[1];
  shadowMat[2][1] = 0.0 - light[1] * ground[2];
  shadowMat[3][1] = 0.0 - light[1] * ground[3];

  shadowMat[0][2] = 0.0 - light[2] * ground[0];
  shadowMat[1][2] = 0.0 - light[2] * ground[1];
  shadowMat[2][2] = dot - light[2] * ground[2];
  shadowMat[3][2] = 0.0 - light[2] * ground[3];

  shadowMat[0][3] = 0.0 - light[3] * ground[0];
  shadowMat[1][3] = 0.0 - light[3] * ground[1];
  shadowMat[2][3] = 0.0 - light[3] * ground[2];
  shadowMat[3][3] = dot - light[3] * ground[3];

/*  glMultMatrixf((const GLfloat *) shadowMat); */
}

/*****************************************************************************/
unsigned short getshort(inf)
     FILE *inf;
{
  unsigned char buf[2];
  
  fread(buf,2,1,inf);
  return (buf[0]<<8)+(buf[1]<<0);
}

/*****************************************************************************/
unsigned short getrgbchar(inf)
     FILE *inf;
{
  unsigned char buf[1];
  
  fread(buf,1,1,inf);
  return (buf[0]);
}

/*****************************************************************************/
unsigned char * read_rgbimage(char *name,int *w, int *h)
{
  unsigned char *image, *temp;
  FILE *image_in;
  unsigned char input_char;
  unsigned short int input_short;
  unsigned char header[512];
  unsigned long int loop;

  if ( (image_in = fopen(name, "rb")) == NULL) 
    {
      return 0;
    }
  input_short=getshort(image_in);
  if (input_short == 0x01da)
    {
      input_char=getrgbchar(image_in);
      if (input_char == 0)
	{
	  input_char=getrgbchar(image_in);
	  input_short=getshort(image_in);
	  if (input_short == 3)
	    {
	      input_short=getshort(image_in);
	      *w=input_short;
	      input_short=getshort(image_in);
	      *h=input_short;
	      image=malloc(*w * *h *4 *sizeof(unsigned char));
	      temp=malloc(*w * *h *sizeof(unsigned char));
	      if ((image == NULL) || (temp == NULL))
		{
		  printf("Error allocating image.\n");
		  exit(0);
		}
	      input_short=getshort(image_in);
	      if (input_short == 4)
		{
		  fread(header,sizeof(unsigned char),500,image_in);
		  fread(temp, sizeof image[0], *w * *h, image_in);
		  for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
		    {
		      image[loop*4+0]=temp[loop];
		    }
		  fread(temp, sizeof image[0], *w * *h, image_in);
		  for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
		    {
		      image[loop*4+1]=temp[loop];
		    }
		  fread(temp, sizeof image[0], *w * *h, image_in);
		  for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
		    {
		      image[loop*4+2]=temp[loop];
		    }
		  fread(temp, sizeof image[0], *w * *h, image_in);
		  for (loop=0;loop<(unsigned long int)(*w * *h);loop++)
		    {
		      image[loop*4+3]=temp[loop];
		    }
		  return image;	
		}
	      else
		{
		  printf("This file isn't a 4 channel RGBA file.\n");
		  exit(0);
		}
	    }	   
	  else
	    {
	      printf("Not a useable RGB file.\n");
	      exit(0);
	    }
	}
      else
	{
	  printf("RLE encoded SGI files are not supported.\n");
	  exit(0);
	}
    }
  else
    {
      printf("File %s doesn't appear to be an SGI rgb file!\n",name);
    }
  return 0;
}
/*****************************************************************************/
unsigned char * read_bwimage(char *name, int *w, int *h)
// From Skyfly.c, Thanks dude
{
    unsigned char   *image;
    FILE            *image_in;
    int             components;

    if ( (image_in = fopen(name, "rb")) == NULL) {
        return 0;
    }

    if (strncmp("terrain", name, 7) == 0) {
         *w = 256;
        *h = 256;
    } else if (strncmp("clouds", name, 6) == 0) {
        *w = 128;
        *h = 128;
    }
    components = 1;

    if (components != 1)
        return 0;

    image = (unsigned char *)malloc(sizeof(unsigned char) * *w * *h);

    fread(image, sizeof image[0], *w * *h, image_in);
    fclose(image_in);
    return image;
}

/*****************************************************************************/
void read_textures(void)
{
 unsigned long loop;

 grass_texture=read_rgbimage(grass_texture_filename,&grass_texture_width,
			     &grass_texture_height);
 grass_side_texture=read_rgbimage(grass_side_texture_filename,
				  &grass_side_texture_width,
				  &grass_side_texture_height);
 grass_top_texture=read_rgbimage(grass_top_texture_filename,
				  &grass_top_texture_width,
				  &grass_top_texture_height);
 pine_texture=read_rgbimage(pine_texture_filename,&pine_texture_width,
			     &pine_texture_height);
 decid_texture=read_rgbimage(decid_texture_filename,&decid_texture_width,
			     &decid_texture_height); 
 eastern_view_texture=read_rgbimage(eastern_view_filename,
				    &eastern_view_texture_width,
				    &eastern_view_texture_height); 
 netrees_texture=read_rgbimage(netrees_filename,
			       &netrees_texture_width,
			       &netrees_texture_height); 
 dirt_texture=read_rgbimage(dirt_filename,
			       &dirt_texture_width,
			       &dirt_texture_height); 
 water_texture=read_rgbimage(water_filename,
			       &water_texture_width,
			       &water_texture_height); 
 beachsand_texture=read_rgbimage(beachsand_filename,
			       &beachsand_texture_width,
			       &beachsand_texture_height); 
 scrub_texture=read_rgbimage(scrub_filename,
			       &scrub_texture_width,
			       &scrub_texture_height); 
 scrubedge_texture=read_rgbimage(scrubedge_filename,
			       &scrubedge_texture_width,
			       &scrubedge_texture_height); 
 south_texture=read_rgbimage(south_filename,
			       &south_texture_width,
			       &south_texture_height); 
 hilledge_texture=read_rgbimage(hilledge_filename,
			       &hilledge_texture_width,
			       &hilledge_texture_height); 
 waves_texture=read_rgbimage(waves_filename,
			       &waves_texture_width,
			       &waves_texture_height); 
 sand_texture=read_rgbimage(sand_filename,
			    &sand_texture_width,
			    &sand_texture_height); 
 outhouse_texture=read_rgbimage(outhouse_filename,
			       &outhouse_texture_width,
			       &outhouse_texture_height); 
 freq_texture=read_rgbimage(freq_filename,
			       &freq_texture_width,
			       &freq_texture_height); 
 ground_texture=read_bwimage(ground_texture_filename,&ground_texture_width,
                          &ground_texture_height);
 sky_texture=read_bwimage(sky_texture_filename,&sky_texture_width,
                          &sky_texture_height);
 for (loop=0;loop<((unsigned long)sky_texture_width*sky_texture_height)-1;
      loop++)
   {
     if (sky_texture[loop] < 25)
       sky_texture[loop]=0;
   }
}

/*****************************************************************************/
void initialize_window(void)
{
 GLfloat mat_specular[]={1.0,1.0,1.0,1.0};
 GLfloat mat_diffuse[]={0.3,0.3,0.3,1.0};
 GLfloat light_position[]={0.0,5000.0,3000.0,0.0};
 GLfloat white_light[]={1.0,1.0,1.0,1.0};
 int style;
 GLfloat ground[4];
 GLfloat light[4];

 ground[0]=0;
 ground[1]=1;
 ground[2]=0;
 ground[3]=0;

 if (location==CAPE_COD)
   {
     light[0]=0.;
     light[1]=5000.;
     light[2]=00.;
     light[3]=0.0;
   }
 else
   {
     light[0]=0.;
     light[1]=5000.;
     light[2]=-3000.;
     light[3]=0.0;
   }
 ShadowMatrix(ground,light,shadow_matrix);

 ground[0]=0;
 ground[1]=1;
 ground[2]=1;
 ground[3]=-100.1;
 ShadowMatrix(ground,light,slope_shadow_matrix);

 ground[0]=0;
 ground[1]=1;
 ground[2]=0;
 ground[3]=-100.1;
 ShadowMatrix(ground,light,top_shadow_matrix);
 style = TUBE_JN_ANGLE | TUBE_JN_CAP;
 gleSetJoinStyle (style);

 glClearColor(0.0,0.0,0.0,1.0);
 if (shading == FLAT)
   {
     glShadeModel(GL_FLAT);
     printf("Using flat shading.\n");
   }
 glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
 glMaterialfv(GL_FRONT,GL_SPECULAR,mat_specular);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_diffuse);
 glLightfv(GL_LIGHT0,GL_POSITION,light_position);
 glLightfv(GL_LIGHT0,GL_DIFFUSE,white_light);
 glLightfv(GL_LIGHT0,GL_SPECULAR,white_light);

 if (use_textures)
   {
     read_textures();

     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glGenTextures(1,&skyTexture);
     glBindTexture(GL_TEXTURE_2D,skyTexture);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexImage2D(GL_TEXTURE_2D,0,1,sky_texture_width,sky_texture_height,0,
		  GL_LUMINANCE,GL_UNSIGNED_BYTE,sky_texture);

     if (location==DAVIS)
       {
     glGenTextures(1,&groundTexture);
     glBindTexture(GL_TEXTURE_2D,groundTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     if (!use_mipmaps)
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	 glTexImage2D(GL_TEXTURE_2D,0,4,ground_texture_width,ground_texture_height,
		      0,GL_ALPHA,GL_UNSIGNED_BYTE,ground_texture);
       }
     else
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	 gluBuild2DMipmaps(GL_TEXTURE_2D,4,ground_texture_width,
			   ground_texture_height,GL_ALPHA,GL_UNSIGNED_BYTE,
			   ground_texture);
       }

     glGenTextures(1,&grassTexture);
     glBindTexture(GL_TEXTURE_2D,grassTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     if (!use_mipmaps)
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,grass_texture_width,
		      grass_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		      grass_texture);
       }
     else
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,grass_texture_width,
			   grass_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
			   grass_texture);
       }

     glGenTextures(1,&grassSideTexture);
     glBindTexture(GL_TEXTURE_2D,grassSideTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     if (!use_mipmaps)
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,grass_side_texture_width,
		  grass_side_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  grass_side_texture);
       }
     else
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,grass_side_texture_width,
			   grass_side_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
			   grass_side_texture);
       }

     glGenTextures(1,&grassTopTexture);
     glBindTexture(GL_TEXTURE_2D,grassTopTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     if (!use_mipmaps)
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	 glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,grass_top_texture_width,
		      grass_top_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		      grass_top_texture);
       }
     else
       {
	 glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	 gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,grass_top_texture_width,
			   grass_top_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
			   grass_top_texture);
       }


     glGenTextures(1,&easternViewTexture);
     glBindTexture(GL_TEXTURE_2D,easternViewTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,eastern_view_texture_width,
		  eastern_view_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  eastern_view_texture);

     glGenTextures(1,&netreesTexture);
     glBindTexture(GL_TEXTURE_2D,netreesTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,netrees_texture_width,
		  netrees_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  netrees_texture);

     glGenTextures(1,&dirtTexture);
     glBindTexture(GL_TEXTURE_2D,dirtTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,dirt_texture_width,
		  dirt_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  dirt_texture);

     glGenTextures(1,&outhouseTexture);
     glBindTexture(GL_TEXTURE_2D,outhouseTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,outhouse_texture_width,
		  outhouse_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  outhouse_texture);

     glGenTextures(1,&freqTexture);
     glBindTexture(GL_TEXTURE_2D,freqTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,freq_texture_width,
		  freq_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  freq_texture);

     glGenTextures(1,&pineTexture);
     glBindTexture(GL_TEXTURE_2D,pineTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,pine_texture_width,
		  pine_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  pine_texture);

     glGenTextures(1,&decidTexture);
     glBindTexture(GL_TEXTURE_2D,decidTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,decid_texture_width,
		  decid_texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		  decid_texture);

       }

     if (location ==CAPE_COD)
       {
     glGenTextures(1,&waterTexture);
     glBindTexture(GL_TEXTURE_2D,waterTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,water_texture_width,
		  water_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  water_texture);

     glGenTextures(1,&beachsandTexture);
     glBindTexture(GL_TEXTURE_2D,beachsandTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,beachsand_texture_width,
		  beachsand_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  beachsand_texture);

     glGenTextures(1,&scrubTexture);
     glBindTexture(GL_TEXTURE_2D,scrubTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,scrub_texture_width,
		  scrub_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  scrub_texture);

     glGenTextures(1,&scrubedgeTexture);
     glBindTexture(GL_TEXTURE_2D,scrubedgeTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,scrubedge_texture_width,
		  scrubedge_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  scrubedge_texture);

     glGenTextures(1,&southTexture);
     glBindTexture(GL_TEXTURE_2D,southTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,south_texture_width,
		  south_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  south_texture);

     glGenTextures(1,&hilledgeTexture);
     glBindTexture(GL_TEXTURE_2D,hilledgeTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,hilledge_texture_width,
		  hilledge_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  hilledge_texture);

     glGenTextures(1,&wavesTexture);
     glBindTexture(GL_TEXTURE_2D,wavesTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,waves_texture_width,
		  waves_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  waves_texture);

     glGenTextures(1,&sandTexture);
     glBindTexture(GL_TEXTURE_2D,sandTexture);
     glPixelStorei(GL_UNPACK_ALIGNMENT,1);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_REPEAT);
     glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_REPEAT);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
     gluBuild2DMipmaps(GL_TEXTURE_2D,GL_RGBA,sand_texture_width,
		  sand_texture_height,GL_RGBA,GL_UNSIGNED_BYTE,
		  sand_texture);
       }

   }

 glEnable(GL_DEPTH_TEST);
 glEnable(GL_LIGHTING);
 glEnable(GL_LIGHT0);

 quadric=gluNewQuadric();
}

/*****************************************************************************/
double gaussrand()
// Thanks to rhoads@paul.rutgers.edu for this code!
// http://remus.rutgers.edu/~rhoads/Code/code.html
{
    static double V2, fac;
    static int phase = 0;
    double S, Z, U1, U2, V1;

    if (phase)
        Z = V2 * fac;
    else
        {
        do {
           U1 = (double)rand() / RAND_MAX;
           U2 = (double)rand() / RAND_MAX;

           V1 = 2 * U1 - 1;
           V2 = 2 * U2 - 1;
           S = V1 * V1 + V2 * V2;
           } while(S >= 1);

        fac = sqrt (-2 * log(S) / S);
        Z = V1 * fac;
        }

    phase = 1 - phase;

    return Z;
}

/*****************************************************************************/
void find_new_thermal_position(float *xpos,float *ypos,int *xcoord,int *ycoord)
{
 float xrand,yrand;
 int num_grid_elements;
 int counter=0;

 num_grid_elements=occupancy_grid_size*occupancy_grid_size;
 xrand=rand()/(RAND_MAX+1.0);
 yrand=rand()/(RAND_MAX+1.0);
 *xcoord=(int)(xrand*occupancy_grid_size);
 *ycoord=(int)(yrand*occupancy_grid_size);
 *xpos=(xrand*occupancy_grid_size*occupancy_grid_res)-(occupancy_grid_size*occupancy_grid_res/2);
 *ypos=(yrand*occupancy_grid_size*occupancy_grid_res)-(occupancy_grid_size*occupancy_grid_res/2);
 if (*xcoord==0)
   {
     (*xcoord)++;                    // Don't create thermals on edges of grid
     *xpos+=occupancy_grid_res;
   }
 if (*xcoord==occupancy_grid_size-1)
   {
     (*xcoord)--;  // the edge of the grid.
     *xpos-=occupancy_grid_res;
   }
 if (*ycoord==0)
   {
     (*ycoord)++;
     *ypos+=occupancy_grid_res;
   }
 if (*ycoord==occupancy_grid_size-1)
   {
     (*ycoord)--;
     *ypos-=occupancy_grid_res;
   }

 while((thermal_occupancy_grid[*xcoord-1][*ycoord-1]!=0 ||
	thermal_occupancy_grid[*xcoord-1][*ycoord]!=0 ||
	thermal_occupancy_grid[*xcoord-1][*ycoord+1]!=0 ||
	thermal_occupancy_grid[*xcoord][*ycoord-1]!=0 ||
	thermal_occupancy_grid[*xcoord][*ycoord]!=0 ||
	thermal_occupancy_grid[*xcoord][*ycoord+1]!=0 ||
	thermal_occupancy_grid[*xcoord+1][*ycoord-1]!=0 ||
	thermal_occupancy_grid[*xcoord+1][*ycoord]!=0 || 
	thermal_occupancy_grid[*xcoord+1][*ycoord+1]!=0) &&
        counter < num_grid_elements)
       {
	 counter++;
	 (*xcoord)++;
	 (*xpos)+=occupancy_grid_res;
	 if (*xcoord ==occupancy_grid_size-1)
	   {
	     *xcoord=1;
	     (*xpos)-=((occupancy_grid_size-2)*occupancy_grid_res);
	     (*ycoord)++;
	     (*ypos)+=occupancy_grid_res;
	     if (*ycoord==occupancy_grid_size-1)
	       {
		 *ycoord=1;
		 *ypos-=((occupancy_grid_size-2)*occupancy_grid_res);
	       }
	   } 
       }
 if (counter == num_grid_elements)
   {
     printf("Thermal density is set way too high!\n");
     *xpos=0;
     *ypos=0;
     *xcoord=occupancy_grid_size/2;
     *ycoord=occupancy_grid_size/2;
   }
}

/*****************************************************************************/
void initialize_wind_field()
//  Initialize thermal positions stregths, radii, etc.
{
 int loop;
 float xpos,ypos;
 int xcoord,ycoord;
 thermal *temp_thermal;
 int xloop,yloop;

 num_thermals=(int)(pow(occupancy_grid_size*occupancy_grid_res,2)*
                    thermal_density);
 for (xloop=0;xloop<occupancy_grid_size;xloop++)
   {
     for(yloop=0;yloop<occupancy_grid_size;yloop++)
       {
	 thermal_occupancy_grid[xloop][yloop]=0;
       }
   }
 for (loop=0;loop<num_thermals;loop++)
   {
     temp_thermal=(thermal *)malloc(sizeof(thermal));
     temp_thermal->next_thermal=thermals;
     thermals=temp_thermal;
     find_new_thermal_position(&xpos,&ypos,&xcoord,&ycoord);
     thermal_occupancy_grid[xcoord][ycoord]=temp_thermal;
     thermals->center_x_position=xpos;
     thermals->center_y_position=ypos;
     thermals->xcoord=xcoord;
     thermals->ycoord=ycoord;
     thermals->radius=(gaussrand()*thermal_radius_sigma)+thermal_radius_mean;
     thermals->boundary_thickness=thermals->radius/5;
     thermals->strength=(gaussrand()*thermal_strength_sigma)+
                        thermal_strength_mean;
     thermals->lifetime=(gaussrand()*thermal_lifetime_sigma)+
                        thermal_lifetime_mean;     
     thermals->time_expired=(rand()/(RAND_MAX+1.0)*thermals->lifetime);
   }
 turb_x_velocity[0]=0;
 turb_y_velocity[0]=0;
 turb_z_velocity[0]=0;
 for (loop=1;loop<turb_grid_size;loop++)
   {
     turb_x_velocity[loop]=turb_x_velocity[loop-1]+(0.03*gaussrand());
     turb_y_velocity[loop]=turb_y_velocity[loop-1]+(0.03*gaussrand());
     turb_z_velocity[loop]=turb_y_velocity[loop-1]+(0.03*gaussrand());
     if (loop==0)
       {
	 printf("%f %f %f\n",turb_x_velocity[0],turb_y_velocity[0],turb_z_velocity[0]);
       }
     turb_x_grad[loop]=gaussrand();
     turb_y_grad[loop]=gaussrand();
     turb_z_grad[loop]=gaussrand();
   }
}

/*****************************************************************************/
void fgSkyVerticesInit( void )
// Thanks to the FlightGear project for the excellent Sky sphere.
{
 float theta;
 int i;

 INNER_RADIUS=distance_to_horizon*sin(30*DEG_TO_RAD);
 MIDDLE_RADIUS=distance_to_horizon*sin(60*DEG_TO_RAD);
 OUTER_RADIUS=distance_to_horizon;
 BOTTOM_RADIUS=distance_to_horizon;
 CENTER_ELEV=distance_to_horizon;
 INNER_ELEV=distance_to_horizon*cos(30*DEG_TO_RAD);
 MIDDLE_ELEV=distance_to_horizon*cos(60*DEG_TO_RAD);
 OUTER_ELEV=0;
 BOTTOM_ELEV=-200;

    for ( i = 0; i < 12; i++ ) {
	theta = (i * 30.0) * DEG_TO_RAD;
	
	inner_vertex[i][0] = cos(theta) * INNER_RADIUS;
	inner_vertex[i][2] = sin(theta) * INNER_RADIUS;
	inner_vertex[i][1] = INNER_ELEV;
        inner_texture[i][0]=(inner_vertex[i][0]/distance_to_horizon)+0.5;
        inner_texture[i][1]=(inner_vertex[i][2]/distance_to_horizon)+0.5;

	middle_vertex[i][0] = cos((double)theta) * MIDDLE_RADIUS;
	middle_vertex[i][2] = sin((double)theta) * MIDDLE_RADIUS;
	middle_vertex[i][1] = MIDDLE_ELEV;
        middle_texture[i][0]=(middle_vertex[i][0]/distance_to_horizon)+0.5;
        middle_texture[i][1]=(middle_vertex[i][2]/distance_to_horizon)+0.5;
	
	outer_vertex[i][0] = cos((double)theta) * OUTER_RADIUS;
	outer_vertex[i][2] = sin((double)theta) * OUTER_RADIUS;
	outer_vertex[i][1] = OUTER_ELEV;
        outer_texture[i][0]=(outer_vertex[i][0]/distance_to_horizon)+0.5;
        outer_texture[i][1]=(outer_vertex[i][2]/distance_to_horizon)+0.5;
	
	bottom_vertex[i][0] = cos((double)theta) * BOTTOM_RADIUS;
	bottom_vertex[i][2] = sin((double)theta) * BOTTOM_RADIUS;
	bottom_vertex[i][1] = BOTTOM_ELEV;
        bottom_texture[i][0]=(bottom_vertex[i][0]/distance_to_horizon)+0.5;
        bottom_texture[i][1]=(bottom_vertex[i][2]/distance_to_horizon)+0.5;
    }
}

/*****************************************************************************/
void draw_sky(void)
// Thanks to the FlightGear project for the excellent Sky sphere.  I've added
// the cloud texture map.
{
 GLfloat inner_color[4];
 GLfloat middle_color[4];
 GLfloat outer_color[4];
 GLfloat sky_color[4];
 GLfloat adj_fog_color[4];
 int i;

 sky_color[0]=0.121;
 sky_color[1]=0.355;
 sky_color[2]=0.637;
 sky_color[3]=1.0;

 inner_color[0]=0.207;
 inner_color[1]=0.457;
 inner_color[2]=0.734;
 inner_color[3]=1.0;

 middle_color[0]=0.441;
 middle_color[1]=0.679;
 middle_color[2]=0.933;
 middle_color[3]=1.0;

 outer_color[0]=0.613;
 outer_color[1]=0.753;
 outer_color[2]=0.933;
 outer_color[3]=1.0;

 adj_fog_color[0]=1.0;
 adj_fog_color[1]=1.0;
 adj_fog_color[2]=1.0;
 adj_fog_color[3]=1.0;

 glPushMatrix();
 glBegin(GL_TRIANGLE_FAN);
 glColor4fv(sky_color);
 glVertex3f(0.0,CENTER_ELEV,0.0);
 for (i=11;i>=0;i--)
  {
   glColor4fv(inner_color);
   glVertex3fv(inner_vertex[i]);
  }
 glColor4fv(inner_color);
 glVertex3fv(inner_vertex[11]);
 glEnd();
 glBegin(GL_TRIANGLE_STRIP);
 for (i=0;i<12;i++)
  {
   glColor4fv(middle_color);
	 glVertex3fv(middle_vertex[i]);
	 glColor4fv(inner_color);
	 glVertex3fv(inner_vertex[i]);
  }
 glColor4fv(middle_color);
 glVertex3fv(middle_vertex[0]);
 glColor4fv(inner_color);
 glVertex3fv(inner_vertex[0]);
 glEnd();
 glBegin(GL_TRIANGLE_STRIP);
 for (i=0;i<12;i++)
  {
   glColor4fv(outer_color);
   glVertex3fv(outer_vertex[i]);
   glColor4fv(middle_color);
   glVertex3fv(middle_vertex[i]);
  }
 glColor4fv(outer_color);
 glVertex3fv(outer_vertex[0]);
 glColor4fv(middle_color);
 glVertex3fv(middle_vertex[0]);
 glEnd();
 glBegin(GL_TRIANGLE_STRIP);
 glColor4fv(outer_color);
 for (i=0;i<12;i++)
  {
   glVertex3fv(bottom_vertex[i]);
   glVertex3fv(outer_vertex[i]);
  }
 glVertex3fv(bottom_vertex[0]);
 glVertex3fv(outer_vertex[0]);
 glEnd();


 if (use_textures)
   {
     glEnable(GL_TEXTURE_2D);
     glEnable(GL_BLEND);
     glBlendFunc(GL_ONE,GL_ONE_MINUS_SRC_COLOR);
     glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_BLEND);
     glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,adj_fog_color);
     glBindTexture(GL_TEXTURE_2D,skyTexture);
     glBegin( GL_TRIANGLE_FAN );
     glColor4fv(sky_color);
     glTexCoord2f(0.5,0.5);
     glVertex3f(0.0,CENTER_ELEV,0.0);
     for (i=11;i>=0;i--)
       {
	 glTexCoord2f(inner_texture[i][0],inner_texture[i][1]);
	 glColor4fv(inner_color);
	 glVertex3fv(inner_vertex[i]);
       }
     glTexCoord2f(inner_texture[11][0],inner_texture[11][1]);
     glColor4fv(inner_color);
     glVertex3fv(inner_vertex[11]);
     glEnd();
     glBegin(GL_TRIANGLE_STRIP);
     for (i=0;i<12;i++)
       {
	 glTexCoord2f(middle_texture[i][0],middle_texture[i][1]);
	 glColor4fv(middle_color);
	 glVertex3fv(middle_vertex[i]);
	 glTexCoord2f(inner_texture[i][0],inner_texture[i][1]);
	 glColor4fv(inner_color);
	 glVertex3fv(inner_vertex[i]);
       }
     glTexCoord2f(middle_texture[0][0],middle_texture[0][1]);
     glColor4fv(middle_color);
     glVertex3fv(middle_vertex[0]);
     glTexCoord2f(inner_texture[0][0],inner_texture[0][1]);
     glColor4fv(inner_color);
     glVertex3fv(inner_vertex[0]);
     glEnd();
     glBegin(GL_TRIANGLE_STRIP);
     for (i=0;i<12;i++)
       {
	 glTexCoord2f(outer_texture[i][0],outer_texture[i][1]);
	 glColor4fv(outer_color);
	 glVertex3fv(outer_vertex[i]);
	 glTexCoord2f(middle_texture[i][0],middle_texture[i][1]);
	 glColor4fv(middle_color);
	 glVertex3fv(middle_vertex[i]);
       }
     glTexCoord2f(outer_texture[0][0],outer_texture[0][1]);
     glColor4fv(outer_color);
     glVertex3fv(outer_vertex[0]);
     glTexCoord2f(middle_texture[0][0],middle_texture[0][1]);
     glColor4fv(middle_color);
     glVertex3fv(middle_vertex[0]);
     glEnd();
     glBegin(GL_TRIANGLE_STRIP);
     glColor4fv(outer_color);
     for (i=0;i<12;i++)
       {
	 glTexCoord2f(bottom_texture[i][0],bottom_texture[i][1]);
	 glVertex3fv(bottom_vertex[i]);
	 glTexCoord2f(outer_texture[i][0],outer_texture[i][2]);
	 glVertex3fv(outer_vertex[i]);
       }
     glTexCoord2f(bottom_texture[0][0],bottom_texture[0][1]);
     glVertex3fv(bottom_vertex[0]);
     glTexCoord2f(outer_texture[0][0],outer_texture[0][1]);
     glVertex3fv(outer_vertex[0]);
     glEnd();
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
   }

 glPopMatrix();
}

/*****************************************************************************/
void draw_medfield_field()
{


}


/*****************************************************************************/
void draw_cape_cod()
{
 GLfloat no_mat[]={0.0,0.0,0.0,0.0};
 GLfloat mat_water[]={.502, 0.650,0.792, 1};
 GLfloat mat_sand[]={.737, 0.694,0.576, 1};
 GLfloat mat_dirtysand[]={.537, 0.494,0.376, 1};
 GLfloat mat_scrub[]={.325, 0.349,0.239, 1};
 GLfloat mat_waves[]={.925, 0.925,0.925, 1};
 GLfloat no_shininess[]={0.0};
 float shift;


  if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_water); // Draw ocean
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_water);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,-0.1,10000.0);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,-0.1,10000.0);
     glEnd();


     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_dirtysand); // Draw beach
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_dirtysand);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,-0.1,150.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_sand); // Draw hill face
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_sand);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-2000.0,100,0.0);
     glNormal3f(0,1,0);
     glVertex3f(2000.0,100,0.0);
     glNormal3f(0,1,0);
     glVertex3f(2000.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glVertex3f(-2000.0,-0.1,100.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_scrub); // Draw hill top
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_scrub);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-2000.0,100,-10000.0);
     glNormal3f(0,1,0);
     glVertex3f(2000.0,100,-10000.0);
     glNormal3f(0,1,0);
     glVertex3f(2000.0,100,0.0);
     glNormal3f(0,1,0);
     glVertex3f(-2000.0,100,0.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_waves); // Draw beach
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_waves);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,3.5,145.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,3.5,145.0);
     glNormal3f(0,1,0);
     glVertex3f(10000.0,3.5,200.0);
     glNormal3f(0,1,0);
     glVertex3f(-10000.0,3.5,200.0);
     glEnd();
   }
 else
   {
     shift=-1*(fmod((double)current_time/500,(double)water_texture_height))/water_texture_height;

     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_water); // Draw parking lot
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_water);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,waterTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-40,-shift);
     glVertex3f(-10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,-shift);
     glVertex3f(10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,40-shift);
     glVertex3f(10000.0,-0.1,10000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-40,40-shift);
     glVertex3f(-10000.0,-0.1,10000.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_sand); // Draw beach lot
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_sand);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,beachsandTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(0,0);
     glVertex3f(0.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,0);
     glVertex3f(10000.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,1);
     glVertex3f(10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0,1);
     glVertex3f(0.0,-0.1,150.0);
     glEnd();

     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(0,0);
     glVertex3f(0.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,0);
     glVertex3f(-10000.0,-0.1,100.0);
     glNormal3f(0,1,0);
     glTexCoord2f(40,1);
     glVertex3f(-10000.0,-0.1,150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0,1);
     glVertex3f(0.0,-0.1,150.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_sand); // Souther horizon
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_sand);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,southTexture);
     glBegin(GL_QUADS);
     glNormal3f(1,0,0);
     glTexCoord2f(0,0);
     glVertex3f(2500.0,-10.1,-430.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0.99,0);
     glVertex3f(2500.0,-20.1,550.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0.99,0.99);
     glVertex3f(2500.0,100.1,550.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0,0.99);
     glVertex3f(2500.0,117.1,-430.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_sand); // Souther horizon
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_sand);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,southTexture);
     glBegin(GL_QUADS);
     glNormal3f(1,0,0);
     glTexCoord2f(0,0);
     glVertex3f(-2500.0,-10.1,-430.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0.99,0);
     glVertex3f(-2500.0,-20.1,550.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0.99,0.99);
     glVertex3f(-2500.0,100.1,550.0);
     glNormal3f(1,0,0);
     glTexCoord2f(0,0.99);
     glVertex3f(-2500.0,117.1,-430.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_waves); // Draw beach lot
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_waves);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,wavesTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(0,0+-5*shift);
     glVertex3f(-10000.0,3.5,145.0);
     glNormal3f(0,1,0);
     glTexCoord2f(50,0+-5*shift);
     glVertex3f(10000.0,3.5,145.0);
     glNormal3f(0,1,0);
     glTexCoord2f(50,1+-5*shift);
     glVertex3f(10000.0,3.5,200.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0,1+-5*shift);
     glVertex3f(-10000.0,3.5,200.0);
     glEnd();



     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_sand); // Hillside
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_sand);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,sandTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,0.707,0.707);
     glTexCoord2f(-128,0);
     glVertex3f(-2000.0,98.8,1.0);
     glNormal3f(0,0.707,0.707);
     glTexCoord2f(128,0);
     glVertex3f(2000.0,98.8,1.0);
     glNormal3f(0,0.707,0.707);
     glTexCoord2f(128,30);
     glVertex3f(2000.0,-0.1,100.0);
     glNormal3f(0,0.707,0.707);
     glTexCoord2f(-128,30);
     glVertex3f(-2000.0,-0.1,100.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_scrub); // Draw hill top
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_scrub);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,scrubTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-70,-70);
     glVertex3f(-2000.0,100,-2000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(70,-70);
     glVertex3f(2000.0,100,-2000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(70,0);
     glVertex3f(2000.0,100,0.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-70,0);
     glVertex3f(-2000.0,100,0.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_scrub); // Draw hill top
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_scrub);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,scrubedgeTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-80,0.98);
     glVertex3f(-2000.0,100.1,-4.0);
     glNormal3f(0,1,0);
     glTexCoord2f(80,0.98);
     glVertex3f(2000.0,100.1,-4.0);
     glNormal3f(0,1,0);
     glTexCoord2f(80,0.02);
     glVertex3f(2000.0,100.1,0.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-80,0.02);
     glVertex3f(-2000.0,100.1,0.0);
     glEnd();

     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_scrub); // Draw hill top
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_scrub);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,hilledgeTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-128,1);
     glVertex3f(-2000.0,100,0.);
     glNormal3f(0,1,0);
     glTexCoord2f(128,1);
     glVertex3f(2000.0,100,0.);
     glNormal3f(0,1,0);
     glTexCoord2f(128,0);
     glVertex3f(2000.0,98.,2.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-128,0);
     glVertex3f(-2000.0,98.,2.0);
     glEnd();

     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
   }
}


/*****************************************************************************/
void draw_davis_field()
{
 GLfloat no_mat[]={0.0,0.0,0.0,0.0};
 GLfloat mat_ground[]={.878, .859, .745, 1};
 GLfloat mat_grass[]={0.458, .657,0.031, 1};
 GLfloat dark_grass_color[]={0.474,0.521,0.272,1};
 GLfloat mat_pine_trees[]={0.37, 0.37,0.37, 1};
 GLfloat mat_deciduous_trees[]={0.517, .561,0.325, 1};
 GLfloat mat_tall_grass[]={0.533,0.6,0.427, 1};
 GLfloat mat_side_tall_grass[]={0.633,0.7,0.527, 1};
 GLfloat mat_log[]={0.509,0.380,0.321, 1};
 GLfloat mat_handi_house[]={0.343,0.622,0.747, 1};
 GLfloat mat_handi_house_roof[]={1.0,1.0,1.0, 1};
 GLfloat mat_trashcan[]={0.225,0.366,0.186, 1};
 GLfloat mat_trashcan_ring[]={0.325,0.466,0.286, 1};
 GLfloat mat_groundhaze[]={1,1,1,1.0};
 GLfloat mat_runway[]={0.20,0.20,0.20,1.0};
 GLfloat fogColor[4]={0.6,0.6,0.6,1.0};
 GLfloat no_shininess[]={0.0};

 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ground); // Draw parking lot
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_ground);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glVertex3f(200.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glVertex3f(200.0,-0.1,-250.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,-250.0);
     glEnd();
   }
 else
   {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_ground); // Draw parking lot
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_ground);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,dirtTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-1.25,-0.625);
     glVertex3f(-100.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glTexCoord2f(2.5,-0.625);
     glVertex3f(200.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glTexCoord2f(2.6,-3.125);
     glVertex3f(200.0,-0.1,-250.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-1.25,-3.125);
     glVertex3f(-100.0,-0.1,-250.0);
     glEnd();
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
   }

 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_grass); // Draw field
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);

     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(400.0,-0.1,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(275.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,-50.0);
     glEnd();
   }
 else
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_grass); // Draw field
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,grassTexture);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-12.5,125);
     glVertex3f(-100.0,-0.1,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(50,125);
     glVertex3f(400.0,-0.1,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(34.375,-6.25);
     glVertex3f(275.0,-0.1,-50.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-12.5,-6.25);
     glVertex3f(-100.0,-0.1,-50.0);
     glEnd();
     glDisable(GL_TEXTURE_2D);
   }

  if (use_textures)
    {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_pine_trees);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_pine_trees);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);
      glColor4f(0,0,0,1.0);
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D,easternViewTexture); 
      glBegin(GL_QUADS);
      glNormal3f(-1,0,0);
      glTexCoord2f(0,0);
      glVertex3f(6000.0,-20,-900.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0.98,0);
      glVertex3f(6000.0,-250,5700.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(6000.0,175,5700.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0,0.98);
      glVertex3f(6000.0,405,-900.0);
      glEnd();
      glBegin(GL_QUADS);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0);
      glVertex3f(-6000.0,-60,-4700.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0);
      glVertex3f(-6000.0,-40,1900.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(-6000.0,405,1900.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0.98);
      glVertex3f(-6000.0,385,-4700.0);
      glEnd();
      glBegin(GL_QUADS);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0);
      glVertex3f(-6000.0,-40,1900.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0);
      glVertex3f(0.0,-40,-6000.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(0.0,405,-6000.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0.98);
      glVertex3f(-6000.0,385,1900.0);
      glEnd();
      glBegin(GL_QUADS);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0);
      glVertex3f(6000.0,-20,-900.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0);
      glVertex3f(0.0,-40,-6000.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(0.0,405,-6000.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0,0.98);
      glVertex3f(6000.0,405,-900.0);
      glEnd();
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);


      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_deciduous_trees);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_deciduous_trees);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);
      glColor4f(0,0,0,1.0);
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D,netreesTexture); 
      glBegin(GL_QUADS);
      glNormal3f(-1,0,0);
      glTexCoord2f(0.98,0);
      glVertex3f(400.0,-10,0.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0.0,0);
      glVertex3f(300.0,-10,-200.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0,0.98);
      glVertex3f(300.0,95,-200.0);
      glNormal3f(-1,0,0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(400.0,95,0.0);
      glEnd();

      glBegin(GL_QUADS);
      glTexCoord2f(0.98,0);
      glVertex3f(-400.0,0,-350.0);
      glTexCoord2f(0.0,0);
      glVertex3f(-250.0,0,-550.0);
      glTexCoord2f(0,0.98);
      glVertex3f(-250.0,85,-550.0);
      glTexCoord2f(0.98,0.98);
      glVertex3f(-400.0,85,-350.0);
      glEnd();


      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }


 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_pine_trees); // Draw pine trees
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_pine_trees);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-700.0,-2,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(-700.0,90,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(550.0,90,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(550.0,-2,1000.0);
     glEnd();
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(550.0,-2,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(550.0,90,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(1000.0,30,950.0);
     glNormal3f(0,1,0);
     glVertex3f(1000.0,-52,950.0);
     glEnd();
   }
 else
   {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_pine_trees); // Draw pine trees
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_pine_trees);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glBindTexture(GL_TEXTURE_2D,pineTexture); 
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-7,0);
     glVertex3f(-700.0,-2,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-7,0.99);
     glVertex3f(-700.0,90,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(5.5,0.99);
     glVertex3f(550.0,90,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(5.5,0);
     glVertex3f(550.0,-2,1000.0);
     glEnd();
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(5.5,0);
     glVertex3f(550.0,-2,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(5.5,0.99);
     glVertex3f(550.0,90,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(10,0.99);
     glVertex3f(1000.0,40,950.0);
     glNormal3f(0,1,0);
     glTexCoord2f(10,0);
     glVertex3f(1000.0,-42,950.0);
     glEnd();
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
   }


 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_deciduous_trees);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_deciduous_trees);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);                                // Draw deciduous trees
     glNormal3f(0,1,0);
     glVertex3f(-200.0,-0.1,50.0);
     glNormal3f(0,1,0);
     glVertex3f(-200.0,40,50.0);
     glNormal3f(0,1,0);
     glVertex3f(-200.0,40,300.0);
     glNormal3f(0,1,0);
     glVertex3f(-200.0,-0.1,300.0);
     glEnd();
   }
 else
   {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_pine_trees);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_pine_trees);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glEnable(GL_TEXTURE_2D);
     glBindTexture(GL_TEXTURE_2D,decidTexture); 
     glBegin(GL_QUADS);                                // Draw deciduous trees
     glNormal3f(0,1,0);
     glTexCoord2f(0.01,0.01);
     glVertex3f(-200.0,-0.1,50.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0.01,0.99);
     glVertex3f(-200.0,40,50.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0.99,0.99);
     glVertex3f(-200.0,40,300.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0.99,0.01);
     glVertex3f(-200.0,-0.1,300.0);
     glEnd();

     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
   }

 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_side_tall_grass);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_side_tall_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,-150.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,2,-150.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,2,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,-0.1,1000.0);
     glEnd();
   }
 else
  {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_side_tall_grass);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_side_tall_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glEnable(GL_TEXTURE_2D);
     glBindTexture(GL_TEXTURE_2D,grassSideTexture); 
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(131.25,0.99);
     glVertex3f(-100.0,-0.1,-150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(131.25,0);
     glVertex3f(-100.0,2,-150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0,0);
     glVertex3f(-100.0,2,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(0,0.99);
     glVertex3f(-100.0,-0.1,1000.0);
     glEnd();
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
  }   


 if (!use_textures)
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_tall_grass);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_tall_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,2,-150.0);
     glNormal3f(0,1,0);
     glVertex3f(-300.0,2,-150.0);
     glNormal3f(0,1,0);
     glVertex3f(-300.0,2,1000.0);
     glNormal3f(0,1,0);
     glVertex3f(-100.0,2,1000.0);
     glEnd();
   }
 else
   {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_tall_grass);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_tall_grass);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glEnable(GL_TEXTURE_2D);
     glColor4f(0,0,0,1.0);
     glEnable(GL_TEXTURE_2D);
     glBindTexture(GL_TEXTURE_2D,grassTopTexture); 
     glBegin(GL_QUADS);
     glNormal3f(0,1,0);
     glTexCoord2f(-10,-5);
     glVertex3f(-100.0,2,-150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-30,-5);
     glVertex3f(-300.0,2,-150.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-30,100);
     glVertex3f(-300.0,2,1000.0);
     glNormal3f(0,1,0);
     glTexCoord2f(-10,100);
     glVertex3f(-100.0,2,1000.0);
     glEnd();   
     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
   }


 if (use_textures)                     // Draw bushes around parking lot
    {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_pine_trees);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_pine_trees);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);
      glEnable(GL_TEXTURE_2D);
      glColor4f(0,0,0,1.0);
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D,decidTexture); 
      glBegin(GL_QUADS);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.01);
      glVertex3f(-105.0,-0.2,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.01);
      glVertex3f(-27.5,-0.2,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.99);
      glVertex3f(-27.5,7,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.99);
      glVertex3f(-105,7,-250.0);
      glEnd();

      glBegin(GL_QUADS);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.01);
      glVertex3f(0.0,-0.2,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.01);
      glVertex3f(215,-0.2,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.99);
      glVertex3f(215,10,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.99);
      glVertex3f(0,10,-250.0);
      glEnd();

      glBegin(GL_QUADS);
      glNormal3f(1,0,0);
      glTexCoord2f(0.01,0.01);
      glVertex3f(-99.9,-0.2,-145.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.99,0.01);
      glVertex3f(-99.9,-0.2,-250.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.99,0.99);
      glVertex3f(-99.9,5,-250.0);
      glNormal3f(1,0,0);
      glTexCoord2f(0.01,0.99);
      glVertex3f(-99.9,5,-145.0);
      glEnd();

      glBegin(GL_QUADS);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.01);
      glVertex3f(200.0,-0.2,-250.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.01);
      glVertex3f(200,-0.2,-100.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.99,0.99);
      glVertex3f(200,6,-100.0);
      glNormal3f(0,0,1);
      glTexCoord2f(0.01,0.99);
      glVertex3f(200,10,-250.0);
      glEnd();

      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }

  if (!use_textures)
    {
      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBegin(GL_QUADS);
      glNormal3f(0,1,0);
      glVertex3f(275.0,-0.1,-50.0);
      glNormal3f(0,1,0);
      glVertex3f(400.0,-0.1,1000.0);
      glNormal3f(0,1,0);
      glVertex3f(400.0,3,1000.0);
      glNormal3f(0,1,0);
      glVertex3f(275.0,3,-50.0);
      glEnd();
    }
  else
    {
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);
      glColor4f(0,0,0,1.0);
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D,grassSideTexture); 
      glBegin(GL_QUADS);
      glNormal3f(0,1,0);
      glTexCoord2f(0,0.99);
      glVertex3f(275.0,-0.1,-50.0);
      glNormal3f(0,1,0);
      glTexCoord2f(88,0.99);
      glVertex3f(400.0,-0.1,1000.0);
      glNormal3f(0,1,0);
      glTexCoord2f(88,0);
      glVertex3f(400.0,3,1000.0);
      glNormal3f(0,1,0);
      glTexCoord2f(0,0);
      glVertex3f(275.0,3,-50.0);
      glEnd();
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0);
    }


 glMaterialfv(GL_FRONT,GL_AMBIENT,mat_log);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_log);
 glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
 glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
 glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
 glPushMatrix();
 glTranslatef(140,0.5,-50.5);
 glRotatef(94,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(97,0.5,-50.5);
 glRotatef(88,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(60,0.5,-50.5);
 glRotatef(90,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(24,0.5,-50.5);
 glRotatef(92,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(-12,0.5,-50.5);
 glRotatef(88,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(-48,0.5,-50.5);
 glRotatef(91,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 glPushMatrix();
 glTranslatef(-84,0.5,-50.5);
 glRotatef(87,0,1,0);
 gluCylinder(quadric,0.5,0.55,35,5,2);
 glPopMatrix();

 if (use_textures)
   {
     glDisable(GL_LIGHTING);
     glDisable(GL_LIGHT0);
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_handi_house);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_handi_house);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glEnable(GL_BLEND);
     glColor4f(0,0,0,1.0);
     glEnable(GL_TEXTURE_2D);
     glBindTexture(GL_TEXTURE_2D,outhouseTexture); 
     glBegin(GL_QUADS);
     glNormal3f(1,0,0);
     glTexCoord2f(0,0);
     glVertex3f(33,0,-38);
     glNormal3f(1,0,0);
     glTexCoord2f(1,0);
     glVertex3f(33,0,-34);
     glNormal3f(1,0,0);
     glTexCoord2f(1,1);
     glVertex3f(33,7.5,-34);
     glNormal3f(1,0,0);
     glTexCoord2f(0,1);
     glVertex3f(33,7.5,-38);
     glNormal3f(0,0,1);
     glTexCoord2f(0,0);
     glVertex3f(33,0,-34);
     glNormal3f(0,0,1);
     glTexCoord2f(1,0);
     glVertex3f(37,0,-34);
     glNormal3f(0,0,1);
     glTexCoord2f(1,1);
     glVertex3f(37,7.5,-34);
     glNormal3f(0,0,1);
     glTexCoord2f(0,1);
     glVertex3f(33,7.5,-34);
     glEnd();

     glDisable(GL_TEXTURE_2D);
     glDisable(GL_BLEND);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0); 
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_handi_house_roof);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_handi_house_roof);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glPushMatrix();
     glTranslatef(35,7.75,-36);
     glScalef(4.25,0.5,4.25);
     glutSolidCube(1);
     glPopMatrix();
   }
 else
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_handi_house);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_handi_house);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glPushMatrix();
     glTranslatef(35,3.75,-36);
     glScalef(1.33,2.5,1.33);
     glutSolidCube(3);
     glPopMatrix();
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_handi_house_roof);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_handi_house_roof);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glPushMatrix();
     glTranslatef(35,7.75,-36);
     glScalef(4.25,0.5,4.25);
     glutSolidCube(1);
     glPopMatrix();
   }

 glMaterialfv(GL_FRONT,GL_AMBIENT,mat_trashcan);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_trashcan);
 glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
 glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
 glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
 glPushMatrix();
 glTranslatef(23,3,-40);
 glRotatef(90,1,0,0);
 gluCylinder(quadric,1.25,1.25,3,10,2);
 glPopMatrix();

 glMaterialfv(GL_FRONT,GL_AMBIENT,mat_trashcan_ring);
 glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_trashcan_ring);
 glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
 glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
 glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
 glPushMatrix();
 glTranslatef(23,1,-40);
 glRotatef(90,1,0,0);
 gluCylinder(quadric,1.3,1.3,0.1,10,2);
 glPopMatrix();
 glPushMatrix();
 glTranslatef(23,2,-40);
 glRotatef(90,1,0,0);
 gluCylinder(quadric,1.3,1.3,0.1,10,2);
 glPopMatrix();


  if (use_textures)   // Draw freq
   {
     glMaterialfv(GL_FRONT,GL_AMBIENT,mat_log);
     glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_log);
     glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
     glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
     glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
     glPushMatrix();
     glTranslatef(17,1,-33.25);
     glScalef(0.25,2,0.25);
     glutSolidCube(1);
     glPopMatrix();
      glDisable(GL_LIGHTING);
      glDisable(GL_LIGHT0);
      glMaterialfv(GL_FRONT,GL_AMBIENT,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_DIFFUSE,mat_side_tall_grass);
      glMaterialfv(GL_FRONT,GL_SPECULAR,no_mat);
      glMaterialfv(GL_FRONT,GL_SHININESS,no_shininess);
      glMaterialfv(GL_FRONT,GL_EMISSION,no_mat);
      glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
      glEnable(GL_BLEND);
      glColor4f(0,0,0,1.0);
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D,freqTexture); 
      glBegin(GL_QUADS);
      glNormal3f(0,1,0);
      glTexCoord2f(0,0);
      glVertex3f(15.0,1,-33.0);
      glNormal3f(0,1,0);
      glTexCoord2f(1,0);
      glVertex3f(18.5,1,-33.0);
      glNormal3f(0,1,0);
      glTexCoord2f(1,1);
      glVertex3f(18.5,6,-33.0);
      glNormal3f(0,1,0);
      glTexCoord2f(0,1);
      glVertex3f(15.0,6,-33.0);
      glEnd();
      glDisable(GL_TEXTURE_2D);
      glDisable(GL_BLEND);
      glEnable(GL_LIGHTING);
      glEnable(GL_LIGHT0); 
   }

}

/****************************************************************************/
void draw_airplane_shadow(float shadow_matrix[4][4])
{
  GLfloat shadow_ambient[] = {0.0, 0.0, 0.0, 1.0};
  GLfloat shadow_diffuse[] = {0.0, 0.0, 0.0, 0.3};
  GLfloat shadow_shiny[] = {0.0};
  GLfloat shadow_specular[] = {0.0, 0.0, 0.0, 1.0};
  extrusion *extrusion_ptr;
  cylinder *cylinder_ptr;
  sphere  *sphere_ptr;
  triangle *triangle_ptr;  
  int loop;

  glMaterialfv(GL_FRONT, GL_AMBIENT, shadow_ambient);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, shadow_diffuse);
  glMaterialfv(GL_FRONT, GL_SPECULAR, shadow_specular);
  glMaterialfv(GL_FRONT, GL_SHININESS, shadow_shiny);
  glColor4f(0,0,0,1.0);
  glPushMatrix();
  glMultMatrixf((const GLfloat *) shadow_matrix);
  extrusion_ptr=extrusions;
  cylinder_ptr=cylinders;
  sphere_ptr=spheres;
  triangle_ptr=triangles;
  glTranslatef(X_cg_rwy,H_cg_rwy, Y_cg_rwy);
  glRotatef(90,0,1,0);
  glRotatef(Psi*RAD_TO_DEG,0,-1,0);
  glRotatef(Theta*RAD_TO_DEG,-1,0,0);
  glRotatef(Phi*RAD_TO_DEG,0,0,1);

  while (extrusion_ptr != NULL)
    {
      glPushMatrix();
      glTranslatef(extrusion_ptr->translate_x,extrusion_ptr->translate_y,
                   extrusion_ptr->translate_z);
      glRotatef(extrusion_ptr->rotate_x,1,0,0);
      glRotatef(extrusion_ptr->rotate_y,0,1,0);
      glRotatef(extrusion_ptr->rotate_z,0,0,1);
      gleSuperExtrusion(extrusion_ptr->num_contour_elements,
                        (gleDouble (*) [2])(*(extrusion_ptr->contour)),
                        (gleDouble (*) [2])(*(extrusion_ptr->
                        contour_normal_vectors)),extrusion_ptr->up,
                        extrusion_ptr->
                        num_points_on_path,(gleDouble (*) [3])
                        (*(extrusion_ptr->point_array)),(float (*) [3])
                        (*(extrusion_ptr->colors)),(gleAffine (*))
                        (*(extrusion_ptr->scaling)));
      glPopMatrix();
      extrusion_ptr=extrusion_ptr->next_extrusion;
    }
  while(cylinder_ptr!=NULL)
    {
      glPushMatrix();
      glTranslatef(cylinder_ptr->translate_x,cylinder_ptr->translate_y,
                   cylinder_ptr->translate_z);
      glRotatef(cylinder_ptr->rotate_x,1,0,0);
      glRotatef(cylinder_ptr->rotate_y,0,1,0);
      glRotatef(cylinder_ptr->rotate_z,0,0,1);
      gluCylinder(quadric,cylinder_ptr->radius_1,cylinder_ptr->radius_2,
		  cylinder_ptr->length,cylinder_ptr->radial_sampling,
		  cylinder_ptr->longitudinal_sampling);
      glPopMatrix();
      cylinder_ptr=cylinder_ptr->next_cylinder;
    }  
  while(sphere_ptr!=NULL)
    {
      glPushMatrix();
      glTranslatef(sphere_ptr->translate_x,sphere_ptr->translate_y,
                   sphere_ptr->translate_z);
      glRotatef(sphere_ptr->rotate_x,1,0,0);
      glRotatef(sphere_ptr->rotate_y,0,1,0);
      glRotatef(sphere_ptr->rotate_z,0,0,1);
      gluSphere(quadric,sphere_ptr->radius,sphere_ptr->slices,
		  sphere_ptr->stacks);
      glPopMatrix();
      sphere_ptr=sphere_ptr->next_sphere;
    }

  glEnable(GL_CULL_FACE);
  while (triangle_ptr!=NULL)
    {
      glPushMatrix();
      glTranslatef(triangle_ptr->translate_x,triangle_ptr->translate_y,
                   triangle_ptr->translate_z);
      glRotatef(triangle_ptr->rotate_x,1,0,0);
      glRotatef(triangle_ptr->rotate_y,0,1,0);
      glRotatef(triangle_ptr->rotate_z,0,0,1);
      glBegin(GL_TRIANGLES);
      for (loop=0;loop<(int)triangle_ptr->num_triangles;loop++)
	{
	  glNormal3f(triangle_ptr->normals[loop][0],
		     triangle_ptr->normals[loop][1],
		     triangle_ptr->normals[loop][2]);
	  glVertex3f(triangle_ptr->vertices[loop][0],
		     triangle_ptr->vertices[loop][1],
		     triangle_ptr->vertices[loop][2]);
	  glVertex3f(triangle_ptr->vertices[loop][3],
		     triangle_ptr->vertices[loop][4],
		     triangle_ptr->vertices[loop][5]);
	  glVertex3f(triangle_ptr->vertices[loop][6],
		     triangle_ptr->vertices[loop][7],
		     triangle_ptr->vertices[loop][8]);
	}
      glEnd();   
      triangle_ptr=triangle_ptr->next_triangle;
      glPopMatrix();
    }
  glDisable(GL_CULL_FACE);

  glPopMatrix();
}
/****************************************************************************/
void draw_airplane()
{
  extrusion *extrusion_ptr;
  cylinder *cylinder_ptr;
  sphere *sphere_ptr;
  triangle *triangle_ptr;
  int loop;
  thermal *thermal_ptr;

  extrusion_ptr=extrusions;
  cylinder_ptr=cylinders;
  sphere_ptr=spheres;
  triangle_ptr=triangles;

 if (training_mode==TRUE)
   {
     thermal_ptr=thermals;
     while(thermal_ptr !=NULL)
       {
	 if ((fabs(thermal_ptr->center_x_position) <2000) && fabs(thermal_ptr->center_y_position)<2000)
	   {
	     glPushMatrix();
	     glColor4f(1,0,0,1);
	     glTranslatef(thermal_ptr->center_x_position,H_cg_rwy, thermal_ptr->center_y_position);
	     gluSphere(quadric,1,3,3);      
	     glPopMatrix();
	   }
	 thermal_ptr=thermal_ptr->next_thermal;
       }
   }

  glPushMatrix();
  glTranslatef(X_cg_rwy,H_cg_rwy, Y_cg_rwy);
  glRotatef(90,0,1,0);
  glRotatef(Psi*RAD_TO_DEG,0,-1,0);
  glRotatef(Theta*RAD_TO_DEG,-1,0,0);
  glRotatef(Phi*RAD_TO_DEG,0,0,1);

  if (training_mode)
    {
      if (stalling)
	{
	  glPushMatrix();
	  glColor4f(1,0,0,1);
	  gluSphere(quadric,0.1,10,10);      
	  glPopMatrix();
	}
    }

  glEnable(GL_COLOR_MATERIAL);
  while (extrusion_ptr != NULL)
    {
      glPushMatrix();
      glTranslatef(extrusion_ptr->translate_x,extrusion_ptr->translate_y,
                   extrusion_ptr->translate_z);
      glRotatef(extrusion_ptr->rotate_x,1,0,0);
      glRotatef(extrusion_ptr->rotate_y,0,1,0);
      glRotatef(extrusion_ptr->rotate_z,0,0,1);
      gleSuperExtrusion(extrusion_ptr->num_contour_elements,
                        (gleDouble (*) [2])(*(extrusion_ptr->contour)),
                        (gleDouble (*) [2])(*(extrusion_ptr->
                        contour_normal_vectors)),extrusion_ptr->up,
                        extrusion_ptr->
                        num_points_on_path,(gleDouble (*) [3])
                        (*(extrusion_ptr->point_array)),(float (*) [3])
                        (*(extrusion_ptr->colors)),(gleAffine (*))
                        (*(extrusion_ptr->scaling)));
      glPopMatrix();
      extrusion_ptr=extrusion_ptr->next_extrusion;
    }
  glDisable(GL_COLOR_MATERIAL);
  while(cylinder_ptr!=NULL)
    {
      glPushMatrix();
      glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,
                   cylinder_ptr->colors);
      glTranslatef(cylinder_ptr->translate_x,cylinder_ptr->translate_y,
                   cylinder_ptr->translate_z);
      glRotatef(cylinder_ptr->rotate_x,1,0,0);
      glRotatef(cylinder_ptr->rotate_y,0,1,0);
      glRotatef(cylinder_ptr->rotate_z,0,0,1);
      gluCylinder(quadric,cylinder_ptr->radius_1,cylinder_ptr->radius_2,
		  cylinder_ptr->length,cylinder_ptr->radial_sampling,
		  cylinder_ptr->longitudinal_sampling);
      glPopMatrix();
      cylinder_ptr=cylinder_ptr->next_cylinder;
    }
  while(sphere_ptr!=NULL)
    {
      glPushMatrix();
      glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE,
                   sphere_ptr->colors);
      glTranslatef(sphere_ptr->translate_x,sphere_ptr->translate_y,
                   sphere_ptr->translate_z);
      glRotatef(sphere_ptr->rotate_x,1,0,0);
      glRotatef(sphere_ptr->rotate_y,0,1,0);
      glRotatef(sphere_ptr->rotate_z,0,0,1);
      gluSphere(quadric,sphere_ptr->radius,sphere_ptr->slices,
		  sphere_ptr->stacks);
      glPopMatrix();
      sphere_ptr=sphere_ptr->next_sphere;
    }

  glEnable(GL_CULL_FACE);
  while (triangle_ptr!=NULL)
    {
      glPushMatrix();
      glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, 
		   triangle_ptr->colors);
      glTranslatef(triangle_ptr->translate_x,triangle_ptr->translate_y,
                   triangle_ptr->translate_z);
      glRotatef(triangle_ptr->rotate_x,1,0,0);
      glRotatef(triangle_ptr->rotate_y,0,1,0);
      glRotatef(triangle_ptr->rotate_z,0,0,1);
      glBegin(GL_TRIANGLES);
      for (loop=0;loop<(int)triangle_ptr->num_triangles;loop++)
	{
	  glNormal3f(triangle_ptr->normals[loop][0],
		     triangle_ptr->normals[loop][1],
		     triangle_ptr->normals[loop][2]);
	  glVertex3f(triangle_ptr->vertices[loop][0],
		     triangle_ptr->vertices[loop][1],
		     triangle_ptr->vertices[loop][2]);
	  glVertex3f(triangle_ptr->vertices[loop][3],
		     triangle_ptr->vertices[loop][4],
		     triangle_ptr->vertices[loop][5]);
	  glVertex3f(triangle_ptr->vertices[loop][6],
		     triangle_ptr->vertices[loop][7],
		     triangle_ptr->vertices[loop][8]);
	}
      glEnd();   
      triangle_ptr=triangle_ptr->next_triangle;
      glPopMatrix();
    }
  glDisable(GL_CULL_FACE);
  glPopMatrix();
}

/*****************************************************************************/
void display(void)
{
 glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 glPushMatrix();
 glLoadIdentity();

 if (location == CAPE_COD)
   {
     gluLookAt(0.0,104.0,-2.5,X_cg_rwy,H_cg_rwy,Y_cg_rwy,
	       0.0,1.0,0.0);
   }
 else
  {
    gluLookAt(0.0,4.0,0.0,X_cg_rwy,H_cg_rwy,Y_cg_rwy,
	      0.0,1.0,0.0);
  }

 glDisable(GL_LIGHTING);
 glDisable(GL_CULL_FACE);
 glDisable(GL_DEPTH_TEST);
 draw_sky();
 glEnable(GL_DEPTH_TEST);
 glEnable(GL_LIGHTING);
 draw_airplane();
 glDisable(GL_NORMALIZE);
 if (location==DAVIS)
   {
     draw_airplane_shadow(shadow_matrix);
     glEnable(GL_LIGHT0);
     glEnable(GL_NORMALIZE);
     draw_davis_field();
   }
 else if (location==MEDFIELD)
   {
     draw_airplane_shadow(shadow_matrix);
     glEnable(GL_LIGHT0);
     glEnable(GL_NORMALIZE);     
     draw_medfield_field();
   }
 else if (location==CAPE_COD)
   {
     if (Y_cg_rwy >100)
       {
	 draw_airplane_shadow(shadow_matrix);
       }
     if (Y_cg_rwy >2)
       {
	 draw_airplane_shadow(slope_shadow_matrix);
       }
     else if (Y_cg_rwy < -2)
       {
	 draw_airplane_shadow(top_shadow_matrix);
       }
     glEnable(GL_LIGHT0);
     glEnable(GL_NORMALIZE);
     draw_cape_cod();
   }
 glPopMatrix();
 glFlush();
 glutSwapBuffers();
}


/*****************************************************************************/
void reshape(int w, int h)
{
 window_xsize=w;
 window_ysize=h;
 glViewport(0,0,(GLsizei)w,(GLsizei) h);
 glMatrixMode(GL_PROJECTION);
 glLoadIdentity();
 gluPerspective(field_of_view,(GLfloat)w/(GLfloat)h,3,9500.0);
 glMatrixMode(GL_MODELVIEW);
}

/*****************************************************************************/
void key(unsigned char key, int px, int py)
{
  switch (key)
    {
      case 27:
        if (input_method==AUDIO) audio_rc_close();
        exit(0);
        break;
      case 'r':
        initialize_flight_model();
        resuming=1;
       break;
    case '+':
      if (zoom_control==KEYBOARD)
	{
	  field_of_view-=5;
	  if (field_of_view < 5)
	    field_of_view=5;
	  glViewport(0,0,(GLsizei)window_xsize,(GLsizei)window_ysize);
	  glMatrixMode(GL_PROJECTION);
	  glLoadIdentity();
	  gluPerspective(field_of_view,
                        (GLfloat)window_xsize/(GLfloat)window_ysize,3,9500.0);
	  glMatrixMode(GL_MODELVIEW);
	}
      break;
    case 't':
      if (training_mode==FALSE)
	{
	  training_mode=TRUE;
	}
      else
	{
	  training_mode=FALSE;
	}
      break;
    case '-':
      if (zoom_control==KEYBOARD)
	{
	  field_of_view+=5;
	  if (field_of_view > 55)
	    field_of_view=55;
	  glViewport(0,0,(GLsizei)window_xsize,(GLsizei)window_ysize);
	  glMatrixMode(GL_PROJECTION);
	  glLoadIdentity();
	  gluPerspective(field_of_view,
                        (GLfloat)window_xsize/(GLfloat)window_ysize,3,9500.0);
	  glMatrixMode(GL_MODELVIEW);
	}
      break;
    }
}

/*****************************************************************************/
void special(int key,int x, int y)
{
  switch(key)
   {
     case GLUT_KEY_UP:
       throttle_input=throttle_input+0.1;
       if (throttle_input>1)
        {
         throttle_input=1;
        }
       break;
     case GLUT_KEY_LEFT:
       rudder_input=rudder_input-0.05;
       if (rudder_input < -0.5)
        {
         rudder_input=-0.5;
        }
       break;
     case GLUT_KEY_RIGHT:
       rudder_input=rudder_input+0.05;
       if (rudder_input > 0.5)
        {
         rudder_input=0.5;
        }
       break;
     case GLUT_KEY_DOWN:
       throttle_input=throttle_input-0.1;
       if (throttle_input<0)
        {
         throttle_input=0;
        }
       break;
   }
}

/*****************************************************************************/
void mouse(int button, int state, int x, int y)
{
 mouse_x=x;
 mouse_y=y;

 switch(button)
  {
   case GLUT_LEFT_BUTTON:
    if (state == GLUT_DOWN)
     glutIdleFunc(idle);
    break;
   case GLUT_RIGHT_BUTTON:
    if (state== GLUT_DOWN)
     glutIdleFunc(NULL);
     resuming=1;
    break;
   default:
    break;
  }
}

/***************************************************************************/
void motion(int x, int y)
{
 if (zoom_control==MOUSE)
   {
     field_of_view=35+(60*(float)(y-(window_ysize/2))/window_ysize);
     glViewport(0,0,(GLsizei)window_xsize,(GLsizei)window_ysize);
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
     gluPerspective(field_of_view,(GLfloat)window_xsize/(GLfloat)window_ysize,3,9500.0);
     glMatrixMode(GL_MODELVIEW);
   }
 if (input_method==MOUSE)
   {
     if (!resuming)
       {
        elevator_input=((float)y/window_ysize)-0.5;
	elevator_input *= -1;
        aileron_input=((float)x/window_xsize)-0.5;
       }
     /*     else
       {
	 elevator_input=0;
	 aileron_input=0;
	 } */
   }
}

/*****************************************************************************/
void joystick(unsigned int button_mask, int x, int y, int z)
{
 if (input_method==JOYSTICK_2)
   {
     aileron_input=(float)x/2000;
     elevator_input=-1*(float)y/2000;
   }
 else if (input_method==JOYSTICK_3)
   {
     aileron_input=(float)x/2000;
     elevator_input=-1*(float)y/2000;
     throttle_input=-1*((float)z/2000)+0.5;
     if (throttle_input < 0.2)
       {
	 throttle_input=0;
       }
   }
}

/*****************************************************************************/
void make_new_thermal(thermal *thermal_ptr)
{
 float xpos,ypos;
 int xcoord,ycoord;
 
 find_new_thermal_position(&xpos,&ypos,&xcoord,&ycoord);
 thermal_occupancy_grid[xcoord][ycoord]=thermal_ptr;
 thermal_ptr->center_x_position=xpos;
 thermal_ptr->center_y_position=ypos;
 thermal_ptr->xcoord=xcoord;
 thermal_ptr->ycoord=ycoord;
 thermal_ptr->radius=(gaussrand()*thermal_radius_sigma)+thermal_radius_mean;
 thermal_ptr->boundary_thickness=thermal_ptr->radius/10;
 thermal_ptr->strength=(gaussrand()*thermal_strength_sigma)+
                     thermal_strength_mean;
 thermal_ptr->lifetime=(gaussrand()*thermal_lifetime_sigma)+
                     thermal_lifetime_mean;     
 thermal_ptr->time_expired=0;
}

/*****************************************************************************/
void update_winds(int delta)
//  Given the time since the last iteration, update the wind field model and
//  set the wind velocities in all three axes in the current position of the
//  aircraft.
{
  thermal *thermal_ptr;
  float x_motion;   // How much has a thermal moved in X in the last timestep
  float y_motion;   // How much has a thermal moved in Y in the last timestep
  float x_wind_velocity,y_wind_velocity;
  int new_xcoord,new_ycoord;

  thermal_ptr=thermals;
  while (thermal_ptr != NULL)
    {
      x_wind_velocity=wind_velocity*cos(M_PI*wind_direction/180);
      y_wind_velocity=wind_velocity*sin(M_PI*wind_direction/180);
      x_motion=((float)delta/1000)*x_wind_velocity;
      y_motion=((float)delta/1000)*y_wind_velocity;
      thermal_ptr->center_x_position+=x_motion;
      thermal_ptr->center_y_position+=y_motion;
      thermal_ptr->time_expired+=(float)delta/1000;
      if ((thermal_ptr->time_expired > thermal_ptr->lifetime) || 
         (thermal_ptr->center_x_position < ((occupancy_grid_size/2)*-1)*
          occupancy_grid_res) || (thermal_ptr->center_x_position >  
          (occupancy_grid_size/2)*occupancy_grid_res) ||
          (thermal_ptr->center_y_position < ((occupancy_grid_size/2)*-1)*
          occupancy_grid_res) || (thermal_ptr->center_y_position >  
          (occupancy_grid_size/2)*occupancy_grid_res))
	{
          thermal_occupancy_grid[thermal_ptr->xcoord][thermal_ptr->ycoord]=0;
	  make_new_thermal(thermal_ptr);
	}
      else
	{
	  new_xcoord=(int)((thermal_ptr->center_x_position+
                          (occupancy_grid_size*occupancy_grid_res/2))/
                          occupancy_grid_res);
	  new_ycoord=(int)((thermal_ptr->center_y_position+
                          (occupancy_grid_size*occupancy_grid_res/2))/
                          occupancy_grid_res);
	  if ((new_xcoord!=thermal_ptr->xcoord) || 
	      (new_ycoord != thermal_ptr->ycoord))
	    {
	      if (thermal_occupancy_grid[new_xcoord][new_ycoord]!=0)
		{
		  printf("Error: multiple thermals in one location!\n");
		}
	      thermal_occupancy_grid[thermal_ptr->xcoord][thermal_ptr->ycoord]=0;
	      thermal_occupancy_grid[new_xcoord][new_ycoord]=thermal_ptr;
	      thermal_ptr->xcoord=new_xcoord;
	      thermal_ptr->ycoord=new_ycoord;
	    }
	}
      thermal_ptr=thermal_ptr->next_thermal;
    }
}

/*****************************************************************************/
void calculate_wind(double X_cg,double Y_cg,double H_cg,double *Vel_north,
		    double *Vel_east, double *Vel_down)
{
  float x_wind_velocity,y_wind_velocity;
  thermal *thermal_ptr;
  int aircraft_xcoord,aircraft_ycoord;
  int xloop,yloop;
  double distance_from_core;
  float total_up_airmass=0;
  float sink_area=0;
  float sink_strength=0;
  float thermal_area=0;
  float lift_area=0;
  int in_thermal=0;
  float angle_in;
  float thermal_wind_x=0;
  float thermal_wind_y=0;
  float v_in_max=0;      // Max velocity of thermal vacuum cleaner wind.

  x_wind_velocity=wind_velocity*cos(M_PI*wind_direction/180);
  y_wind_velocity=wind_velocity*sin(M_PI*wind_direction/180);
 
  aircraft_xcoord=(int)((X_cg+(occupancy_grid_size*occupancy_grid_res/2))/
                        occupancy_grid_res);
  aircraft_ycoord=(int)((Y_cg+(occupancy_grid_size*occupancy_grid_res/2))/
                          occupancy_grid_res);
  if ((aircraft_xcoord > area_of_influence) &&
      (aircraft_xcoord < occupancy_grid_size-area_of_influence-1) &&
      (aircraft_ycoord > area_of_influence) &&
      (aircraft_ycoord < occupancy_grid_size-area_of_influence-1))
    {
      for (xloop=(-1*area_of_influence);xloop<=area_of_influence;xloop++)
	{
	  for (yloop=(-1*area_of_influence);yloop<=area_of_influence;yloop++)
	    {
	      if (thermal_occupancy_grid[aircraft_xcoord+xloop]
                                        [aircraft_ycoord+yloop] != 0)
		{
		  thermal_ptr=thermal_occupancy_grid[aircraft_xcoord+xloop]
                                                    [aircraft_ycoord+yloop];
		  thermal_area=(M_PI*thermal_ptr->radius*thermal_ptr->radius);
		  lift_area+=thermal_area;
		  total_up_airmass+=thermal_area*thermal_ptr->strength;
		}
	    }
	}

      sink_area=((2*area_of_influence+1)*(2*area_of_influence+1)*
                 occupancy_grid_res*occupancy_grid_res)-lift_area;
      sink_strength= total_up_airmass/sink_area;
      for (xloop=(-1*area_of_influence);xloop<=area_of_influence;xloop++)
	{
	  for (yloop=(-1*area_of_influence);yloop<=area_of_influence;yloop++)
	    {
	      if (thermal_occupancy_grid[aircraft_xcoord+xloop]
                                        [aircraft_ycoord+yloop] != 0)
		{
		  thermal_ptr=thermal_occupancy_grid[aircraft_xcoord+xloop]
                                                    [aircraft_ycoord+yloop];
		  distance_from_core=sqrt(((X_cg-thermal_ptr->center_x_position)*(X_cg-thermal_ptr->center_x_position))+((Y_cg-thermal_ptr->center_y_position)*(Y_cg-thermal_ptr->center_y_position)));
		  if (H_cg < 1000)
		    {
		      v_in_max=thermal_ptr->strength*thermal_ptr->radius/100;
		      if (distance_from_core > thermal_ptr->radius)
			{
			  v_in_max/=pow(distance_from_core/thermal_ptr->radius,2);
			}
		      else
			{
			  v_in_max*=distance_from_core/thermal_ptr->radius;
			}
		      angle_in=atan2((thermal_ptr->center_y_position-Y_cg),(thermal_ptr->center_x_position-X_cg));
		      if (H_cg < 50)
			{
			  thermal_wind_x+=v_in_max*cos(angle_in);
			  thermal_wind_y+=v_in_max*sin(angle_in);
			}
		      else if (H_cg<1000)
			{
			  thermal_wind_x+=(v_in_max*cos(angle_in))*((950-(H_cg-50))/950);
			  thermal_wind_y+=(v_in_max*sin(angle_in))*((950-(H_cg-0))/950);
			}
		    }
		  if (distance_from_core < thermal_ptr->radius)
		    {
		      *Vel_down= -1*thermal_ptr->strength;
		      in_thermal=TRUE;
		    }
		  else if (distance_from_core < thermal_ptr->radius+thermal_ptr->boundary_thickness)
		    {
		      *Vel_down= -1*thermal_ptr->strength + (thermal_ptr->strength+sink_strength)*((distance_from_core-thermal_ptr->radius)/thermal_ptr->boundary_thickness);
		      in_thermal=TRUE;
		    }
		}
	    }
	}
      if (!in_thermal)
	{
	  *Vel_down= sink_strength;
	}
      if (H_cg < 50)
	{
	  *Vel_down *=(H_cg/50);
	}
      if ((location == CAPE_COD) && (dynamic_soaring==FALSE))
	{
	  if ((Y_cg <140) && (Y_cg > -40))
	    {
	      *Vel_down-=(0.4*wind_velocity*pow(sin((Y_cg+40)*3.14159/180),2));
	      if (H_cg<100)
		{
		}
	      else if ((H_cg >= 100) && (H_cg < 250))
		{
		  *Vel_down*=(1.0-((H_cg-100)/150));
		}
 	    }
	 }
      else if ((location ==CAPE_COD) && (dynamic_soaring == TRUE))
	{
	  if ((Y_cg > 0) && (H_cg < 100))
	    {
	      if (H_cg >= 86)
		{
		 y_wind_velocity*=(H_cg-93)/7;
		}
	      else if (H_cg < 86)
		{
		  y_wind_velocity*=-1;
		}
	    }
	}
      *Vel_east= y_wind_velocity+thermal_wind_y;
      *Vel_north= x_wind_velocity+thermal_wind_x;
    }
}

/*****************************************************************************/
void idle(void)
{
  int delta;
  int multiloop;
  double V_north_x,V_east_x,V_down_x;   // temp vars for calculating wind grad.
  double V_north_y,V_east_y,V_down_y;
  double V_north_z,V_east_z,V_down_z;
  double delta_space=0.1;

  current_time = glutGet(GLUT_ELAPSED_TIME);
  if (resuming)
    {
      initialization_time=current_time;
      time_after_last_integration=0;
      resuming=0;
    }
  if (input_method==RCTRAN)
    {
	  if ((radio_type == FUTABA) || (radio_type == HITEC))
	  {
		aileron_input=(((float)rctran[0]/(cpu_speed*1000))-1.5);  
		elevator_input=(((float)rctran[1]/(cpu_speed*1000))-1.5);
		rudder_input=(((float)rctran[3]/(cpu_speed*1000))-1.5);   
		throttle_input=(((float)rctran[2]/(cpu_speed*1000))-1.0);
	  }
	  else if (radio_type == AIRTRONICS)
	  {
		aileron_input=(((float)rctran[1]/(cpu_speed*1000))-1.5); 
		elevator_input=(((float)rctran[0]/(cpu_speed*1000))-1.5);
		rudder_input=(((float)rctran[3]/(cpu_speed*1000))-1.5);   
		throttle_input=-1*(((float)rctran[2]/(cpu_speed*1000))-2.0);
	  }
	  else if (radio_type == JR)
	  {
		aileron_input=(((float)rctran[1]/(cpu_speed*1000))-1.5);  
		elevator_input=(((float)rctran[2]/(cpu_speed*1000))-1.5);
		rudder_input=(((float)rctran[3]/(cpu_speed*1000))-1.5);   
		throttle_input=-1*(((float)rctran[0]/(cpu_speed*1000))-2.0);
	  }
     if (throttle_input < 0.2)
       throttle_input=0;
     brake_input=(((float)rctran[4]/(cpu_speed*1000))-1.0);    
    }
 else if (input_method==PARALLEL1)
   {
     get_data_from_parallel_interface(rc_channel_values);
   }
  else if (input_method==PARALLEL2)
    {
     get_data_from_parallel_interface(rc_channel_values);
    }
  else if (input_method==PARALLEL3)
    {
     get_data_from_parallel_interface(rc_channel_values);
    }
  else if (input_method==AUDIO)
    {
     nb_rc_channel_values = get_data_from_audio_interface(rc_channel_values);
    }
  if ((input_method == PARALLEL1) || (input_method == PARALLEL2) ||
      (input_method == PARALLEL3) || (input_method == AUDIO))
    {
	  if (radio_type == FUTABA)
	  {
                aileron_input=rc_channel_values[0];
		elevator_input=rc_channel_values[1];
		rudder_input=rc_channel_values[3];
		throttle_input=rc_channel_values[2]+0.5;
	  }
	  else if (radio_type == AIRTRONICS)
	  {
                aileron_input=rc_channel_values[1];
		elevator_input=rc_channel_values[0];
		rudder_input=rc_channel_values[3];
		throttle_input=rc_channel_values[2]+0.5;
	  }
	  else if (radio_type == HITEC)
	  {
                aileron_input=rc_channel_values[0];
		elevator_input=rc_channel_values[1];
		rudder_input=rc_channel_values[3];
		throttle_input=rc_channel_values[2]+0.5;
	  }
	  else if (radio_type == JR)
	  {
                aileron_input=rc_channel_values[1];
		elevator_input=rc_channel_values[2];
		rudder_input=rc_channel_values[3];
		throttle_input=rc_channel_values[0]+0.5;
	  }
	  else if (radio_type == COCKPIT)
	  {
                aileron_input=rc_channel_values[0];
		elevator_input=rc_channel_values[1];
		rudder_input=rc_channel_values[2];
		throttle_input=rc_channel_values[5]+0.5;
	  }
     if (throttle_input > 0.95)
       {
	 throttle_input=1.0;
       }
     else if (throttle_input<0.05)
       {
	 throttle_input=0.0;
       }
              // printf("ail%f elev%f rudd%f thr%f\n",aileron_input,elevator_input,rudder_input,throttle_input);
    }
 delta = (current_time-initialization_time) - time_after_last_integration;
 multiloop=(int)(((float)delta/1000)/dt);
 time_after_last_integration+=multiloop*dt*1000;
 update_winds(delta);
 calculate_wind(X_cg_rwy+delta_space,Y_cg_rwy,H_cg_rwy,&V_north_x,
		&V_east_x,&V_down_x);
 calculate_wind(X_cg_rwy,Y_cg_rwy+delta_space,H_cg_rwy,&V_north_y,
		&V_east_y,&V_down_y);
 calculate_wind(X_cg_rwy,Y_cg_rwy,H_cg_rwy+delta_space,&V_north_z,
		&V_east_z,&V_down_z);
 calculate_wind(X_cg_rwy,Y_cg_rwy,H_cg_rwy,&V_north_airmass,
		&V_east_airmass,&V_down_airmass);

 U_atmo_X=(V_north_x-V_north_airmass)/delta_space;
 U_atmo_Y=(V_north_y-V_north_airmass)/delta_space;
 U_atmo_Z=(V_north_z-V_north_airmass)/delta_space;
 V_atmo_X=(V_east_x-V_east_airmass)/delta_space;
 V_atmo_Y=(V_east_y-V_east_airmass)/delta_space;
 V_atmo_Z=(V_east_z-V_east_airmass)/delta_space;
 W_atmo_X=(V_down_x-V_down_airmass)/delta_space;
 W_atmo_Y=(V_down_y-V_down_airmass)/delta_space;
 W_atmo_Z=(V_down_z-V_down_airmass)/delta_space;
 ls_update(multiloop);
 glutPostRedisplay();
}

/*****************************************************************************/
int main(int argc,char **argv)
{
  char game_mode_string[32];  //Stores the string describing the video mode

#if defined(__APPLE__) || defined(MACOSX)
  glutInit(&argc, argv);
  read_config_file("crrcsim.cfg");    // Read CRRCsim configuration from disk
  read_airplane_file(argc,argv);      // Read current airplane from disk
  sprintf(game_mode_string,"%ix%i:%i@%i",x_resolution,y_resolution,
          color_depth,refresh_rate);
#else
  read_config_file("crrcsim.cfg");    // Read CRRCsim configuration from disk
  read_airplane_file(argc,argv);      // Read current airplane from disk
  sprintf(game_mode_string,"%ix%i:%i@%i",x_resolution,y_resolution,
          color_depth,refresh_rate);
  glutInit(&argc, argv);
#endif  // __APPLE__

  glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
  if (fullscreen)
   {
    glutGameModeString(game_mode_string);
    glutEnterGameMode();
    printf("Starting fullscreen in video mode %s\n",game_mode_string);
   }
  else
   {
    glutInitWindowSize(x_resolution,y_resolution);
    if (location==DAVIS)
      {
	glutCreateWindow("CRRCSim: Charles River RC Flight Simulator, Davis Field, Sudbury, Massachusetts");
      }
    else if (location==MEDFIELD)
      {
	glutCreateWindow("CRRCSim: Charles River RC Flight Simulator, Medfield Flying Field, Medfield, Massachusetts");
      }
    else if (location=CAPE_COD)
      {
	glutCreateWindow("CRRCSim: Charles River RC Flight Simulator, Duck Harbor, Cape Cod, Massachusetts");
      }
   }

  if (input_method==RCTRAN)
   {
#ifdef linux
    rctran=(volatile unsigned long long *)mbuff_attach("crrcsim",
           sizeof(volatile unsigned long long)*8);
#endif
    if (rctran==NULL)
     {
      printf("Mapping shared memory for RC transmitter interface failed.\n");
      printf("Reverting to mouse input.\n");
      input_method=MOUSE;
     }
   }
  initialize_window();
  initialize_wind_field();
  initialize_graphics();
  initialize_flight_model();
  glutDisplayFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(key);
  glutSpecialFunc(special);
  glutMouseFunc(mouse);
  if (input_method == MOUSE || zoom_control==MOUSE)
   {
    glutMotionFunc(motion);
    glutPassiveMotionFunc(motion);
   }
  if (input_method==JOYSTICK_2 || input_method==JOYSTICK_3 ||
           input_method == JOYSTICK_4)
    {
      glutJoystickFunc(joystick,30);
    }
  glutIdleFunc(idle);
  glutMainLoop();
  return 0;
}












